from abjad.tools import durationtools


def multiply_and_scale_contents_of_measures_in_expr(expr, multiplier_pairs):
    '''.. versionadded:: 1.1

    Multiply and scale contents of measures in `expr` by `multiplier_pairs`.

    The `multiplier_pairs` argument must be a list of 
    ``(contents_multiplier, denominator_multiplier)`` pairs.

    Both `contents_multiplier` and `denominator_multiplier` must be positive integers.

    Example 1. Multiply measure contents by ``3``. Scale time signature denominator by ``3``::

        >>> measure = Measure((3, 16), "c'16 c'16 c'16")

    ::

        >>> measuretools.multiply_and_scale_contents_of_measures_in_expr(measure, [(3, 3)])
        [Measure(9/48, [c'32, c'32, c'32, c'32, c'32, c'32, c'32, c'32, c'32])]

    Example 2. Multiply measure contents by ``3``. Scale time signature denominator by ``2``::

        >>> measure = Measure((3, 16), "c'16 c'16 c'16")

    ::

        >>> measuretools.multiply_and_scale_contents_of_measures_in_expr(measure, [(3, 2)])
        [Measure(9/32, [c'32, c'32, c'32, c'32, c'32, c'32, c'32, c'32, c'32])]

    Example 3. Multiply measure contents ``3``::

        >>> measure = Measure((3, 16), "c'16 c'16 c'16")

    ::

        >>> measuretools.multiply_and_scale_contents_of_measures_in_expr(measure, [(3, 1)])
        [Measure(9/16, [c'16, c'16, c'16, c'16, c'16, c'16, c'16, c'16, c'16])]

    Return list of measures changed.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import measuretools

    assert isinstance(multiplier_pairs, list)
    assert all([isinstance(pair, tuple) for pair in multiplier_pairs])

    result = []
    num_pairs = len(multiplier_pairs)
    for i, measure in enumerate(iterationtools.iterate_measures_in_expr(expr)):
        concentration_pair = multiplier_pairs[i % num_pairs]
        assert isinstance(concentration_pair, tuple)
        spin_count, scalar_denominator = concentration_pair
        measuretools.multiply_contents_of_measures_in_expr(measure, spin_count)
        multiplier = durationtools.Duration(1, scalar_denominator)
        measuretools.scale_measure_and_adjust_time_signature(measure, multiplier)
        result.append(measure)

    return result



from abjad.tools import iterationtools
from abjad.tools import sequencetools
from abjad.tools.wellformednesstools.Check import Check


class DuplicateIdCheck(Check):

    def _run(self, expr):
        violators = []
        components = iterationtools.iterate_components_in_expr(expr)
        total_ids = [id(x) for x in components]
        unique_ids = sequencetools.truncate_runs_in_sequence(total_ids)
        if len(unique_ids) < len(total_ids):
            for current_id in unique_ids:
                if 1 < total_ids.count(current_id):
                    violators.extend([x for x in components if id(x) == current_id])
        return violators, len(total_ids)



from abjad import *


def test_labeltools_label_leaves_in_expr_with_pitch_class_numbers_01():
    '''With number = True.'''

    t = Staff("c'8 d'8 e'8 f'8")
    labeltools.label_leaves_in_expr_with_pitch_class_numbers(t, number=True)

    r'''
    \new Staff {
        c'8
            _ \markup {
                \small
                    0
                }
        d'8
            _ \markup {
                \small
                    2
                }
        e'8
            _ \markup {
                \small
                    4
                }
        f'8
            _ \markup {
                \small
                    5
                }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t0\n\t\t\t}\n\td'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t2\n\t\t\t}\n\te'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t4\n\t\t\t}\n\tf'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t5\n\t\t\t}\n}"


def test_labeltools_label_leaves_in_expr_with_pitch_class_numbers_02():
    '''With color = True.'''

    t = Staff("c'8 d'8 e'8 f'8")
    labeltools.label_leaves_in_expr_with_pitch_class_numbers(t, number=False, color=True)

    r'''
    \new Staff {
      \once \override NoteHead #'color = #(x11-color 'red)
      c'8
      \once \override NoteHead #'color = #(x11-color 'orange)
      d'8
      \once \override NoteHead #'color = #(x11-color 'ForestGreen)
      e'8
      \once \override NoteHead #'color = #(x11-color 'MediumOrchid)
      f'8
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\t\\once \\override NoteHead #'color = #(x11-color 'red)\n\tc'8\n\t\\once \\override NoteHead #'color = #(x11-color 'orange)\n\td'8\n\t\\once \\override NoteHead #'color = #(x11-color 'ForestGreen)\n\te'8\n\t\\once \\override NoteHead #'color = #(x11-color 'MediumOrchid)\n\tf'8\n}"



from abjad.tools import iterationtools
from abjad.tools.wellformednesstools.Check import Check


class MissingParentCheck(Check):
    '''Each node except the root needs a parent.
    '''

    def _run(self, expr):
        violators = []
        total = 0
        for component in iterationtools.iterate_components_in_expr(expr):
            total += 1
            if component is not expr:
                if component.parent is None:
                    violators.append(component)
        return violators, total



from abjad import *


def test_MetricalHierarchyInventory_storage_format_01():

    inventory = timesignaturetools.MetricalHierarchyInventory([(4, 4), (3, 4)])

    r'''
    timesignaturetools.MetricalHierarchyInventory([
        timesignaturetools.MetricalHierarchy(
            '(4/4 (1/4 1/4 1/4 1/4))'
            ),
        timesignaturetools.MetricalHierarchy(
            '(3/4 (1/4 1/4 1/4))'
            )
        ])
    '''

    assert inventory.storage_format == "timesignaturetools.MetricalHierarchyInventory([\n\ttimesignaturetools.MetricalHierarchy(\n\t\t'(4/4 (1/4 1/4 1/4 1/4))'\n\t\t),\n\ttimesignaturetools.MetricalHierarchy(\n\t\t'(3/4 (1/4 1/4 1/4))'\n\t\t)\n\t])"





from abjad import *


def test_MetricalHierarchy___init___01():

    time_signature = contexttools.TimeSignatureMark((2, 4))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(2/4 (1/4 1/4))'


def test_MetricalHierarchy___init___02():

    time_signature = contexttools.TimeSignatureMark((3, 4))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(3/4 (1/4 1/4 1/4))'


def test_MetricalHierarchy___init___03():

    time_signature = contexttools.TimeSignatureMark((4, 4))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(4/4 (1/4 1/4 1/4 1/4))'


def test_MetricalHierarchy___init___04():

    time_signature = contexttools.TimeSignatureMark((6, 8))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(6/8 ((3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8))))'


def test_MetricalHierarchy___init___05():

    time_signature = contexttools.TimeSignatureMark((5, 8))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(5/8 ((3/8 (1/8 1/8 1/8)) (2/8 (1/8 1/8))))'


def test_MetricalHierarchy___init___06():

    time_signature = contexttools.TimeSignatureMark((12, 4))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(12/4 ((3/4 (1/4 1/4 1/4)) (3/4 (1/4 1/4 1/4)) (3/4 (1/4 1/4 1/4)) (3/4 (1/4 1/4 1/4))))'


def test_MetricalHierarchy___init___07():

    time_signature = contexttools.TimeSignatureMark((1, 4))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(1/4 (1/4))'


def test_MetricalHierarchy___init___08():

    time_signature = contexttools.TimeSignatureMark((10, 4))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(10/4 ((5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)))) (5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4))))))'


def test_MetricalHierarchy___init___09():

    time_signature = contexttools.TimeSignatureMark((11, 4))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(11/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4))))'


def test_MetricalHierarchy___init___10():

    time_signature = contexttools.TimeSignatureMark((25, 4))
    assert timesignaturetools.MetricalHierarchy(time_signature).rtm_format == \
        '(25/4 ((15/4 ((5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)))) (5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)))) (5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)))))) (10/4 ((5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)))) (5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4))))))))'


def test_MetricalHierarchy___init___11():

    hierarchy = timesignaturetools.MetricalHierarchy('(4/4 ((2/4 (1/4 1/4)) (2/4 (1/4 1/4))))')



# TODO: can this be made public? Perhaps bound to a class?
def _get_stem_tremolo_format_contributions(component):
    '''.. versionadded:: 2.0
    '''
    from abjad.tools import marktools

    result = []

    if marktools.is_component_with_stem_tremolo_attached(component):
        stem_tremolo = marktools.get_stem_tremolo_attached_to_component(component)
        result.append(stem_tremolo.format)

    return ['stem tremolo', result]



from abjad.tools import contexttools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools._WindInstrument import _WindInstrument


class Flute(_WindInstrument):
    r'''.. versionadded:: 2.0

    Abjad model of the flute::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> instrumenttools.Flute()(staff)
        Flute()(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Flute }
            \set Staff.shortInstrumentName = \markup { Fl. }
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    The flute targets staff context by default.
    '''

    def __init__(self, **kwargs):
        _WindInstrument.__init__(self, **kwargs)
        self._default_instrument_name = 'flute'
        self._default_performer_names.extend(['flautist', 'flutist'])
        self._default_short_instrument_name = 'fl.'
        self._is_primary_instrument = True
        self.sounding_pitch_of_written_middle_c = pitchtools.NamedChromaticPitch("c'")
        self.primary_clefs = [contexttools.ClefMark('treble')]
        self._copy_primary_clefs_to_all_clefs()
        self._traditional_pitch_range = pitchtools.PitchRange(0, 38)



def is_singleton(expr):
    r'''.. versionadded:: 2.9
    
    True when `expr` is a tuple of length ``1``::

        >>> sequencetools.is_singleton((19,))
        True

    Otherwise false::

        >>> sequencetools.is_singleton((19, 20, 21))
        False

    Return boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == 1



import os
import re
from abjad.tools import configurationtools


def get_last_output_file_name():
    '''Get last output file name like ``6222.ly``.

    Return string.
    '''
    from abjad import ABJCFG

    pattern = re.compile('\d{4,4}.[a-z]{2,3}')
    all_file_names = os.listdir(ABJCFG['abjad_output'])
    all_output = [fn for fn in all_file_names if pattern.match(fn)]
    if all_output == []:
        last_output_file_name = None
    else:
        last_output_file_name = sorted(all_output)[-1]
    return last_output_file_name



'''Abjad ``pitchtools`` package.

    Dependencies:

    The ``pitchtools`` package may import ``sequencetools`` at top level.

    The ``pitchtools`` package should not import any component packages 
    at top level.

    The ``pitchtools`` package should not import ``contexttools`` at
    top level, and vice versa.

'''
from abjad.tools import importtools

importtools.import_structured_package(__path__[0], globals(), package_root_name='abjad')

_documentation_section = 'core'



def destroy_spanners_attached_to_components_in_expr(expr, klass=None):
    r'''.. versionadded:: 2.9

    Destroy spanners of `klass` attached to components in `expr`::

        >>> staff = Staff("c'4 [ ( d' e' f' ) ]")

    ::

        >>> f(staff)
        \new Staff {
            c'4 [ (
            d'4
            e'4
            f'4 ] )
        }

    ::

        >>> spanners = spannertools.destroy_spanners_attached_to_components_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    Return tuple of zero or more empty spanners.

    Order of spanners in return value can not be predicted.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import spannertools

    result = []
    for component in iterationtools.iterate_components_in_expr(expr):
        result.extend(spannertools.destroy_spanners_attached_to_component(component, klass=klass))

    return tuple(result)



from abjad.tools.spannertools.Spanner import Spanner


class PianoPedalSpanner(Spanner):
    r'''Abjad piano pedal spanner:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spannertools.PianoPedalSpanner(staff[:])
        PianoPedalSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sustainOn
            d'8
            e'8
            f'8 \sustainOff
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return piano pedal spanner.
    '''

    ### CLASS ATTRIBUTES ###

    _kinds = {'sustain': (r'\sustainOn', r'\sustainOff'),
            'sostenuto':(r'\sostenutoOn', r'\sostenutoOff'),
            'corda': (r'\unaCorda', r'\treCorde')}

    _styles = ['text', 'bracket', 'mixed']

    ### INITIALIZER ###

    def __init__(self, components=None):
        Spanner.__init__(self, components)
        self.kind = 'sustain'
        self.style = 'mixed'

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new.kind = self.kind
        new.style = self.style

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r"\set Staff.pedalSustainStyle = #'%s" % self.style)
        return result

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(self._kinds[self.kind][0])
        if self._is_my_last_leaf(leaf):
            result.append(self._kinds[self.kind][1])
        return result

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def kind():
        def fget(self):
            r'''Get piano pedal spanner kind::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(staff[:])
                >>> spanner.kind
                'sustain'

            Set piano pedal spanner kind::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(staff[:])
                >>> spanner.kind = 'sostenuto'
                >>> spanner.kind
                'sostenuto'

            Acceptable values ``'sustain'``, ``'sostenuto'``, ``'corda'``.
            '''
            return self._kind
        def fset(self, arg):
            if not arg in self._kinds.keys():
                raise ValueError("Type must be in %s" % self._kinds.keys())
            self._kind = arg
        return property(**locals())

    @apply
    def style():
        def fget(self):
            r'''Get piano pedal spanner style::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(staff[:])
                >>> spanner.style
                'mixed'

            Set piano pedal spanner style::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(staff[:])
                >>> spanner.style = 'bracket'
                >>> spanner.style
                'bracket'

            Acceptable values ``'mixed'``, ``'bracket'``, ``'text'``.
            '''
            return self._style
        def fset(self, arg):
            if not arg in self._styles:
                raise ValueError("Style must be in %s" % self._styles)
            self._style = arg
        return property(**locals())



from abjad.tools import iotools
from abjad.tools.documentationtools import AbjadAPIGenerator
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript
import os


class BuildApiScript(DeveloperScript):
    '''Build the Abjad APIs:

    ::

        bash$ ajv api -h
        usage: build-api [-h] [--version] [-M] [-X] [-C] [--format FORMAT]

        Build the Abjad APIs.

        optional arguments:
          -h, --help          show this help message and exit
          --version           show program's version number and exit
          -M, --mainline      build the mainline API
          -X, --abjad.tools  build the abjad.tools API
          -C, --clean         run "make clean" before building the api
          --format FORMAT     Sphinx builder to use

    Return `BuildApiScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'api'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return None

    @property
    def short_description(self):
        return 'Build the Abjad APIs.'

    @property
    def version(self):
        return 1.0

    ### PRIVATE METHODS ###

    def _build_experimental_api(self, format='html', clean=False):

        from abjad import ABJCFG

        class ExperimentalAPIGenerator(AbjadAPIGenerator):

            _api_title = 'Abjad Experimental API'

            @property
            def docs_api_index_path(self):
                return os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'docs', 'source', 'index.rst')

            @property
            def path_definitions(self):
                from abjad import ABJCFG
                return (
                    (
                        os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'tools'),
                        os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'docs', 'source', 'tools'),
                        'experimental.tools.',
                    ),
                    (
                        os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'demos'),
                        os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'docs', 'source', 'demos'),
                        'experimental.demos.',
                    ),
                )

            @property
            def root_package(self):
                return 'experimental'

            @property
            def tools_package_path_index(self):
                return 2

        ExperimentalAPIGenerator()(verbose=True)

        # print greeting
        print 'Now building the Experimental {} docs ...'.format(format.upper())
        print ''

        # change to docs directory because makefile lives there
        docs_directory = os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'docs')
        os.chdir(docs_directory)

        # optionally, make clean before building
        if clean:
            print 'Cleaning build directory ...'
            iotools.spawn_subprocess('make clean')

        # make html docs
        iotools.spawn_subprocess('make {}'.format(format))

    def _build_mainline_api(self, format='html', clean=False):

        from abjad import ABJCFG

        AbjadAPIGenerator()(verbose=True)

        # print greeting
        print 'Now building the {} docs ...'.format(format.upper())
        print ''

        # change to docs directory because makefile lives there
        docs_directory = os.path.relpath(os.path.join(ABJCFG.ABJAD_PATH, 'docs'))
        os.chdir(docs_directory)

        # optionally, make clean before building
        if clean:
            print 'Cleaning build directory ...'
            iotools.spawn_subprocess('make clean')

        # make html docs
        iotools.spawn_subprocess('make {}'.format(format))

    ### PUBLIC METHODS ###

    def process_args(self, args):
        format = args.format
        clean = args.clean
        if args.mainline:
            self._build_mainline_api(format=format, clean=clean)
        if args.experimental:
            self._build_experimental_api(format=format, clean=clean)

    def setup_argument_parser(self, parser):

        parser.add_argument('-M', '--mainline',
            action='store_true',
            help='build the mainline API'
            )

        parser.add_argument('-X', '--experimental',
            action='store_true',
            help='build the abjad.tools API'
            )

        parser.add_argument('-C', '--clean',
            action='store_true',
            dest='clean',
            help='run "make clean" before building the api',
            )

        parser.add_argument('--format',
            choices=('html', 'latex', 'latexpdf'),
            dest='format',
            help='Sphinx builder to use',
            metavar='FORMAT',
            )

        parser.set_defaults(format='html')



from abjad import *


def test_tietools_add_or_remove_tie_chain_notes_to_achieve_scaled_written_duration_01():
    '''Change trivial tie chain to nontrivial tie chain.
    '''

    staff = Staff("c'8 [ ]")
    tie_chain = tietools.get_tie_chain(staff[0])
    tietools.add_or_remove_tie_chain_notes_to_achieve_scaled_written_duration(tie_chain, Fraction(5, 4))

    r'''
    \new Staff {
        c'8 [ ~
        c'32 ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\tc'8 [ ~\n\tc'32 ]\n}"


def test_tietools_add_or_remove_tie_chain_notes_to_achieve_scaled_written_duration_02():
    '''Change nontrivial tie chain to trivial tie chain.
    '''

    staff = Staff("c'8 [ ~ c'32 ]")
    tie_chain = tietools.get_tie_chain(staff[0])
    tietools.add_or_remove_tie_chain_notes_to_achieve_scaled_written_duration(tie_chain, Fraction(4, 5))

    r'''
    \new Staff {
        c'8 [ ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\tc'8 [ ]\n}"



def timespan_2_starts_when_timespan_1_starts(timespan_1=None, timespan_2=None, hold=False):
    r'''.. versionadded:: 2.11

    Make time relation template indicating that expression starts when timespan starts::

        >>> timerelationtools.timespan_2_starts_when_timespan_1_starts()
        TimespanTimespanTimeRelation('timespan_1.start == timespan_2.start')

    Return boolean or time relation.
    '''
    from abjad.tools import timerelationtools

    time_relation = timerelationtools.TimespanTimespanTimeRelation(
        'timespan_1.start == timespan_2.start',
        timespan_1=timespan_1,
        timespan_2=timespan_2)

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



import collections
import fractions
from abjad.tools import contexttools
from abjad.tools import lilypondfiletools
from abjad.tools import pitchtools
from abjad.tools import schemetools
from abjad.tools import scoretools
from abjad.tools import stafftools


def make_polyphonic_percussion_score_from_nonoverlapping_trees(trees, colorkey=None):
    '''Make a polyphonic percussion score from a collections of non-overlapping trees.

    Return LilyPondFile.
    '''

    from abjad.tools import timeintervaltools

    if isinstance(trees, timeintervaltools.TimeIntervalTreeDictionary):
        trees = trees.values()
    else:
        assert isinstance(trees, collections.Iterable) and len(trees) and \
            all([isinstance(tree, timeintervaltools.TimeIntervalTree) for tree in trees])
    

    pitches = pitchtools.make_n_middle_c_centered_pitches(len(trees))
    voices = []
    for zipped in zip(trees, pitches):
        tree = zipped[0]
        pitch = zipped[1]
        voice = timeintervaltools.make_voice_from_nonoverlapping_intervals(tree,
            colorkey = colorkey, pitch = pitch)
        voices.append(voice)

    staff = stafftools.Staff(voices)
    staff.is_parallel = True
    staff.override.staff_symbol.line_count = len(voices)
    contexttools.ClefMark('percussion')(staff)

    score = scoretools.Score([staff])
    score.override.glissando.thickness = 5
    score.override.note_head.style = 'harmonic'
    score.override.rest.transparent = True
    score.override.spacing_spanner.strict_note_spacing = True
    score.override.spacing_spanner.uniform_stretching = True
    score.override.glissando.breakable = True
    score.set.proportional_notation_duration = schemetools.SchemeMoment(fractions.Fraction(1, 32))
    padding = 0.5
    bound_details = schemetools.Scheme(
        ('right',
            schemetools.SchemePair('attach-dir', 0),
            schemetools.SchemePair('padding', padding)),
        ('left',
            schemetools.SchemePair('attach-dir', 0),
            schemetools.SchemePair('padding', padding)),
        quoting="'")
    score.override.glissando.bound_details = bound_details

    lily = lilypondfiletools.make_basic_lilypond_file(score)
    lily.default_paper_size = ('11x17', 'landscape')
    lily.paper_block.ragged_right = True

    return lily



def compute_logical_and_of_intervals(intervals):
    '''Compute the logical AND of a collection of intervals.

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return tree

    depth_tree = timeintervaltools.compute_depth_of_intervals(tree)
    logic_tree = timeintervaltools.TimeIntervalTree([x for x in depth_tree if 1 < x['depth']])

    return logic_tree




def fracture_spanners_attached_to_component(component, direction=None, klass=None):
    r'''.. versionadded:: 1.1

    Fracture all spanners attached to `component` according to `direction`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beam = beamtools.BeamSpanner(staff.leaves)
        >>> slur = spannertools.SlurSpanner(staff.leaves)
        >>> trill = spannertools.TrillSpanner(staff)
        >>> f(staff)
        \new Staff {
            c'8 [ ( \startTrillSpan
            d'8
            e'8
            f'8 ] ) \stopTrillSpan
        }

    ::

        >>> parts = spannertools.fracture_spanners_attached_to_component(staff[1], Right)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ( \startTrillSpan
            d'8 ] )
            e'8 [ (
            f'8 ] ) \stopTrillSpan
        }

    Set `direction` to ``Left``, ``Right`` or ``None``.
    '''

    result = []
    for spanner in component.spanners:
        if klass is None:
            result.append(spanner.fracture(spanner.index(component), direction))
        elif isinstance(spanner, klass):
            result.append(spanner.fracture(spanner.index(component), direction))
    result.sort()
    return result



from abjad.tools import containertools


def parse_rtm_syntax(rtm):
    r'''Parse RTM syntax:

    ::

        >>> rtm = '(1 (1 (1 (1 1)) 1))'
        >>> rhythmtreetools.parse_rtm_syntax(rtm)
        FixedDurationTuplet(1/4, [c'8, c'16, c'16, c'8])

    Also supports fractional durations:

    ::

        >>> rtm = '(3/4 (1 1/2 (4/3 (1 -1/2 1))))'
        >>> rhythmtreetools.parse_rtm_syntax(rtm)
        FixedDurationTuplet(3/16, [c'8, c'16, {@ 15:8 c'8, r16, c'8 @}])
        >>> f(_)
        \fraction \times 9/17 {
            c'8
            c'16
            \times 8/15 {
                c'8
                r16
                c'8
            }
        }

    Return `FixedDurationTuplet` or `Container` instance.
    '''
    from abjad.tools import rhythmtreetools

    result = rhythmtreetools.RhythmTreeParser()(rtm)

    con = containertools.Container()

    for node in result:
        tuplet = node((1, 4))
        # following line added 2012-08-01. tb.
        tuplet = tuplet[0]
        if tuplet.is_trivial:
            con.extend(tuplet[:])
        else:
            con.append(tuplet)

    if len(con) == 1:
        return con[0]
    return con



import collections
import itertools
import numbers
from abjad.tools import durationtools
from abjad.tools import mathtools


def millisecond_pitch_pairs_to_q_events(pairs):
    '''Convert a list of pairs of millisecond durations and pitches to
    a list of ``QEvent`` instances.

    Pitch values must be one of the following:

    1. A single chromatic pitch number, indicating a note,
    2. None, indicating a silence, or
    3. An iterable of chromatic pitch numbers, indicating a chord.

    ::

        >>> durations = [1001, 503, 230, 1340]
        >>> pitches = [None, 0, (1, 2, 3), 4.5]
        >>> pairs = zip(durations, pitches)
        >>> for x in quantizationtools.millisecond_pitch_pairs_to_q_events(pairs): x
        ...
        quantizationtools.SilentQEvent(
            durationtools.Offset(0, 1),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1001, 1),
            (NamedChromaticPitch("c'"),),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1504, 1),
            (NamedChromaticPitch("cs'"), NamedChromaticPitch("d'"), NamedChromaticPitch("ef'")),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1734, 1),
            (NamedChromaticPitch("eqs'"),),
            attachments=()
            )
        quantizationtools.TerminalQEvent(
            durationtools.Offset(3074, 1)
            )

    Return a list of ``QEvent`` instances.
    '''
    from abjad.tools import quantizationtools

    # validate input
    assert isinstance(pairs, collections.Iterable)
    assert all([isinstance(x, collections.Iterable) for x in pairs])
    assert all([len(x) == 2 for x in pairs])
    assert all([0 < x[0] for x in pairs])
    for pair in pairs:
        assert isinstance(pair[1], (numbers.Number, type(None), collections.Iterable))
        if isinstance(pair[1], collections.Iterable):
            assert 0 < len(pair[1])
            assert all([isinstance(x, numbers.Number) for x in pair[1]])

    # fuse silences
    g = itertools.groupby(pairs, lambda x: x[1] is not None)
    groups = []
    for value, group in g:
        if value:
            groups.extend(list(group))
        else:
            duration = sum([x[0] for x in group])
            groups.append((duration, None))

    # find offsets
    offsets = mathtools.cumulative_sums_zero([abs(x[0]) for x in groups])

    # build QEvents
    q_events = []
    for pair in zip(offsets, groups):

        offset = durationtools.Offset(pair[0])
        pitches = pair[1][1]

        if isinstance(pitches, collections.Iterable):
            assert all([isinstance(x, numbers.Number) for x in pitches])
            q_events.append(quantizationtools.PitchedQEvent(offset, pitches))
        elif isinstance(pitches, type(None)):
            q_events.append(quantizationtools.SilentQEvent(offset))
        elif isinstance(pitches, numbers.Number):
            q_events.append(quantizationtools.PitchedQEvent(offset, [pitches]))

    q_events.append(quantizationtools.TerminalQEvent(durationtools.Offset(offsets[-1])))

    return q_events



from abjad.tools import chordtools
from abjad.tools import notetools
from abjad.tools import resttools


def color_leaf(leaf, color):
    r'''.. versionadded:: 2.0

    Color note::

        >>> note = Note("c'4")

    ::

        >>> labeltools.color_leaf(note, 'red')
        Note("c'4")

    ::

        >>> f(note)
        \once \override Accidental #'color = #red
        \once \override Dots #'color = #red
        \once \override NoteHead #'color = #red
        c'4

    ::

        >>> show(note) # doctest: +SKIP

    Color rest::

        >>> rest = Rest('r4')

    ::

        >>> labeltools.color_leaf(rest, 'red')
        Rest('r4')

    ::

        >>> f(rest)
        \once \override Dots #'color = #red
        \once \override Rest #'color = #red
        r4

    ::

        >>> show(rest) # doctest: +SKIP

    Color chord::

        >>> chord = Chord("<c' e' bf'>4")

    ::

        >>> labeltools.color_leaf(chord, 'red')
        Chord("<c' e' bf'>4")

    ::

        >>> f(chord)
        \once \override Accidental #'color = #red
        \once \override Dots #'color = #red
        \once \override NoteHead #'color = #red
        <c' e' bf'>4

    ::

        >>> show(chord) # doctest: +SKIP

    Return `leaf`.
    '''

    # color leaf
    if isinstance(leaf, (notetools.Note, chordtools.Chord)):
        leaf.override.accidental.color = color
        leaf.override.dots.color = color
        leaf.override.note_head.color = color
    elif isinstance(leaf, resttools.Rest):
        leaf.override.dots.color = color
        leaf.override.rest.color = color

    # return leaf
    return leaf



from abjad import *


def test_ComplexGlissandoSpanner_01():
    t = Staff("c'8 d'8 r8 e'8")
    gliss = spannertools.ComplexGlissandoSpanner(t.leaves[:])
    assert isinstance(gliss, spannertools.ComplexGlissandoSpanner)

    '''
    \new Staff {
        c'8 \glissando
        d'8 \glissando
        \once \override NoteColumn #'glissando-skip = ##t
        \once \override Rest #'transparent = ##t
        r8
        e'8
    }
    '''

    assert t.lilypond_format == "\\new Staff {\n\tc'8 \\glissando\n\td'8 \\glissando\n\t\\once \\override NoteColumn #'glissando-skip = ##t\n\t\\once \\override Rest #'transparent = ##t\n\tr8\n\te'8\n}"



from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.contexttools.ContextMark import ContextMark


class TimeSignatureMark(ContextMark):
    r'''.. versionadded:: 2.0

    Abjad model of a time signature::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> contexttools.TimeSignatureMark((4, 8))(staff[0])
        TimeSignatureMark((4, 8))(c'8)

    ::

        >>> f(staff)
        \new Staff {
            \time 4/8
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Abjad time signature marks target **staff context** by default.

    Initialize time signature marks to **score context** like this::

        >>> contexttools.TimeSignatureMark((4, 8), target_context=Score)
        TimeSignatureMark((4, 8), target_context=Score)

    .. note:: add more initializer examples.

    .. note:: add example of `suppress` keyword.

    .. note:: turn `suppress` into managed attribute.

    Return time signature object.
    '''

    ### CLASS ATTRIBUTES ###

    _default_positional_input_arguments = ((4, 8), )

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        from abjad.tools.stafftools.Staff import Staff
        target_context = kwargs.get('target_context', None)
        ContextMark.__init__(self, target_context=target_context)
        if self.target_context is None:
            self._target_context = Staff
        if self._target_context == Staff:
            self._has_default_target_context = True
        else:
            self._has_default_target_context = False

        partial, suppress = None, None

        # initialize numerator and denominator from *args
        if len(args) == 1 and isinstance(args[0], type(self)):
            time_signature = args[0]
            numerator, denominator = time_signature.numerator, time_signature.denominator
            partial = time_signature.partial
            suppress = time_signature.suppress
        elif len(args) == 1 and isinstance(args[0], durationtools.Duration):
            numerator, denominator = args[0].numerator, args[0].denominator
        elif len(args) == 1 and isinstance(args[0], tuple):
            numerator, denominator = args[0][0], args[0][1]
        elif len(args) == 1 and hasattr(args[0], 'numerator') and hasattr(args[0], 'denominator'):
            numerator, denominator = args[0].numerator, args[0].denominator 
        else:
            raise TypeError('invalid time_signature initialization: {!r}.'.format(args))
        self._numerator = numerator
        self._denominator = denominator

        # initialize partial from **kwargs
        partial = partial or kwargs.get('partial', None)
        if not isinstance(partial, (type(None), durationtools.Duration)):
            raise TypeError
        self._partial = partial
        if partial is not None:
            self._partial_repr_string = ', partial=%r' % self._partial
        else:
            self._partial_repr_string = ''

        # initialize suppress from kwargs
        suppress = suppress or kwargs.get('suppress', None)
        if not isinstance(suppress, (bool, type(None))):
            raise TypeError
        self.suppress = suppress

        # initialize derived attributes
        self._multiplier = self.implied_prolation
        self._has_non_power_of_two_denominator = not mathtools.is_nonnegative_integer_power_of_two(
            self.denominator)

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        return type(self)((self.numerator, self.denominator),
            partial = self.partial, target_context = self.target_context)

    # note that this can not be defined on superclass
    __deepcopy__ = __copy__

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            return self.numerator == arg.numerator and self.denominator == arg.denominator
        elif isinstance(arg, tuple):
            return self.numerator == arg[0] and self.denominator == arg[1]
        else:
            return False

    def __ge__(self, arg):
        if isinstance(arg, type(self)):
            return self.duration >= arg.duration
        else:
            raise TypeError

    # TODO: define __getnewargs__?

    def __gt__(self, arg):
        if isinstance(arg, type(self)):
            return self.duration > arg.duration
        else:
            raise TypeError

    def __le__(self, arg):
        if isinstance(arg, type(self)):
            return self.duration <= arg.duration
        else:
            raise TypeError

    def __lt__(self, arg):
        if isinstance(arg, type(self)):
            return self.duration < arg.duration
        else:
            raise TypeError

    def __ne__(self, arg):
        return not self == arg

    def __nonzero__(self):
        return True

    def __repr__(self):
        if self._has_default_target_context:
            return '%s((%s, %s)%s)%s' % (type(self).__name__, self.numerator,
                self.denominator, self._partial_repr_string, self._attachment_repr_string)
        else:
            return '%s((%s, %s)%s, target_context=%s)%s' % (
                type(self).__name__, self.numerator,
                self.denominator, self._partial_repr_string, self._target_context_name, 
                self._attachment_repr_string)

    def __str__(self):
        return '%s/%s' % (self.numerator, self.denominator)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return '%s/%s' % (self.numerator, self.denominator)

    @property
    def _keyword_argument_names(self):
        return (
            'partial',
            'suppress',
            )

    @property
    def _positional_argument_values(self):
        return (self.pair, )

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def duration(self):
        r'''Time signature mark duration::

            >>> contexttools.TimeSignatureMark((3, 8)).duration
            Duration(3, 8)

        Return duration.
        '''
        return durationtools.Duration(self.numerator, self.denominator)

    @property
    def effective_context(self):
        r'''Time signature mark effective context.

        Return none when time signature mark is not yet attached::

            >>> time_signature = contexttools.TimeSignatureMark((3, 8))

        ::

            >>> time_signature.effective_context is None
            True

        Return context when time signature mark is attached::

            >>> staff = Staff()
            >>> time_signature.attach(staff)
            TimeSignatureMark((3, 8))(Staff{})

        ::

            >>> time_signature.effective_context 
            Staff{}

        Return context or none.
        '''
        return ContextMark.effective_context.fget(self)

    @property
    def has_non_power_of_two_denominator(self):
        r'''True when time signature mark has non-power-of-two denominator::

            >>> contexttools.TimeSignatureMark((7, 12)).has_non_power_of_two_denominator
            True

        Otherwise false::

            >>> contexttools.TimeSignatureMark((3, 8)).has_non_power_of_two_denominator
            False

        Return boolean.
        '''
        return self._has_non_power_of_two_denominator

    @property
    def implied_prolation(self):
        '''.. versionadded:: 2.11

        Time signature mark implied prolation.

        Example 1. Implied prolation of time signature with power-of-two denominator::

            >>> contexttools.TimeSignatureMark((3, 8)).implied_prolation
            Multiplier(1, 1)
        
        Example 2. Implied prolation of time signature with non-power-of-two denominator::

            >>> contexttools.TimeSignatureMark((7, 12)).implied_prolation
            Multiplier(2, 3)

        Return multiplier.
        '''
        dummy_duration = durationtools.Duration(1, self.denominator)
        return dummy_duration.implied_prolation

    @property
    def lilypond_format(self):
        r'''Time signature mark LilyPond format::

            >>> contexttools.TimeSignatureMark((3, 8)).lilypond_format
            '\\time 3/8'

        Return string.
        '''
        if self.suppress:
            return []
        elif self.partial is None:
            return r'\time %s/%s' % (self.numerator, self.denominator)
        else:
            result = []
            duration_string = self.partial.lilypond_duration_string
            partial_directive = r'\partial %s' % duration_string
            result.append(partial_directive)
            result.append(r'\time %s/%s' % (self.numerator, self.denominator))
            return result

    @property
    def pair(self):
        '''.. versionadded:: 2.8

        Time signature numerator / denominator pair::

            >>> contexttools.TimeSignatureMark((3, 8)).pair
            (3, 8)

        Return pair.
        '''
        return (self.numerator, self.denominator)

    @property
    def start_component(self):
        r'''Time signature mark start component.

        Return none when time signature mark is not yet attached::

            >>> time_signature = contexttools.TimeSignatureMark((3, 8))
            >>> time_signature.start_component is None
            True

        Return component when time signature mark is attached::

            >>> staff = Staff()
            >>> time_signature.attach(staff)
            TimeSignatureMark((3, 8))(Staff{})

        ::

            >>> time_signature.start_component
            Staff{}

        Return component or none.
        '''
        return ContextMark.start_component.fget(self)

    @property
    def storage_format(self):
        r'''Time signature mark storage format::

            >>> print contexttools.TimeSignatureMark((3, 8)).storage_format
            contexttools.TimeSignatureMark(
                (3, 8)
                )

        Return string.
        '''
        return ContextMark.storage_format.fget(self)

    @property
    def target_context(self):
        r'''Time signature mark target context::

            >>> contexttools.TimeSignatureMark((3, 8)).target_context
            <class 'abjad.tools.stafftools.Staff.Staff.Staff'>

        Time signature marks target the staff context by default.

        This can be changed at initialization.
        
        Return class.
        '''
        return ContextMark.target_context.fget(self)

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def denominator():
        def fget(self):
            r'''Get denominator of time signature mark::

                >>> time_signature = contexttools.TimeSignatureMark((3, 8))

            ::

                >>> time_signature.denominator
                8

            Set denominator of time signature mark::

                >>> time_signature.denominator = 16

            ::

                >>> time_signature.denominator
                16

            Return integer.
            '''
            return self._denominator
        def fset(self, denominator):
            assert isinstance(denominator, int)
            self._denominator = denominator
        return property(**locals())

    @apply
    def numerator():
        def fget(self):
            '''Get numerator of time signature mark::

                >>> time_signature = contexttools.TimeSignatureMark((3, 8))
                >>> time_signature.numerator
                3

            Set numerator of time signature mark::

                >>> time_signature.numerator = 4
                >>> time_signature.numerator
                4

            Set integer.
            '''
            return self._numerator
        def fset(self, numerator):
            assert isinstance(numerator, int)
            self._numerator = numerator
        return property(**locals())

    @apply
    def partial():
        def fget(self):
            '''Get partial measure pick-up of time signature mark::

                >>> time_signature = contexttools.TimeSignatureMark(
                ...     (3, 8), partial=Duration(1, 8))
                >>> time_signature.partial
                Duration(1, 8)

            Set partial measure pick-up of time signature mark::

                >>> time_signature.partial = Duration(1, 4)
                >>> time_signature.partial
                Duration(1, 4)

            Set duration or none.
            '''
            return self._partial
        def fset(self, partial):
            if partial is not None:
                partial = durationtools.Duration(partial)
            self._partial = partial
        return property(**locals())

    ### PUBLIC METHODS ###

    # Time signature marks do not check for other conflicting time signature marks at attachment.
    # The reason for this is that voodoo is being done to time signature marks elsewhere in the code.
    # This is less than optimal and someday time signature marks should check for conflicts at attachment.
    def attach(self, start_component):
        r'''Attach time signature mark to `start_component`::

            >>> time_signature = contexttools.TimeSignatureMark((3, 8))
            >>> staff = Staff()

        ::

            >>> time_signature.attach(staff)
            TimeSignatureMark((3, 8))(Staff{})

        Return time signature mark.
        '''
        from abjad.tools import contexttools
        from abjad.tools import marktools
        klasses = (type(self), )
        if contexttools.is_component_with_context_mark_attached(start_component, klasses):
            raise ExtraMarkError('component already has context mark attached.')
        return marktools.Mark.attach(self, start_component)

    def detach(self):
        r'''Detach time signature mark::

            >>> time_signature = contexttools.TimeSignatureMark((3, 8))
            >>> staff = Staff()

        ::

            >>> time_signature.attach(staff)
            TimeSignatureMark((3, 8))(Staff{})

        ::

            >>> time_signature.detach()
            TimeSignatureMark((3, 8))

        Return time signature mark.
        '''
        return ContextMark.detach(self)

    def with_power_of_two_denominator(self, contents_multiplier=durationtools.Multiplier(1)):
        r'''Create new time signature equivalent to current
        time signature with power-of-two denominator.

            >>> time_signature = contexttools.TimeSignatureMark((3, 12))

        ::

            >>> time_signature.with_power_of_two_denominator()
            TimeSignatureMark((2, 8))

        Return new time signature mark.
        '''
        # check input
        contents_multiplier = durationtools.Multiplier(contents_multiplier)
    
        # save non_power_of_two time_signature and denominator
        non_power_of_two_denominator = self.denominator

        # find power_of_two denominator
        if contents_multiplier == durationtools.Multiplier(1):
            power_of_two_denominator = mathtools.greatest_power_of_two_less_equal(
                non_power_of_two_denominator)
        else:
            power_of_two_denominator = mathtools.greatest_power_of_two_less_equal(
                non_power_of_two_denominator, 1)

        # find power_of_two pair
        non_power_of_two_pair = mathtools.NonreducedFraction(self.pair)
        power_of_two_pair = non_power_of_two_pair.with_denominator(power_of_two_denominator)

        # return new power_of_two time signature mark
        return type(self)(power_of_two_pair)



from abjad.tools import datastructuretools


def test_TreeContainer___eq___01():

    a = datastructuretools.TreeContainer([])
    b = datastructuretools.TreeContainer([])

    assert a == b


def test_TreeContainer___eq___02():

    a = datastructuretools.TreeContainer([
        datastructuretools.TreeNode()
        ])
    b = datastructuretools.TreeContainer([
        datastructuretools.TreeNode()
        ])

    assert a == b


def test_TreeContainer___eq___03():

    a = datastructuretools.TreeContainer([])
    b = datastructuretools.TreeContainer([
        datastructuretools.TreeNode()
        ])
    c = datastructuretools.TreeContainer([
        datastructuretools.TreeNode(),
        datastructuretools.TreeNode()
        ])

    assert a != b
    assert a != c
    assert b != c




from abjad import *


def test_Spanner___contains___01():

    class MockSpanner(spannertools.Spanner):
        def __init__(self, components=None):
            spannertools.Spanner.__init__(self, components)
        def _copy_keyword_args(self, new):
            pass

    note = Note("c'4")

    spanner = MockSpanner([Note("c'4")])

    assert not note in spanner



def get_spanners_attached_to_component(component, klass=None):
    r'''.. versionadded:: 2.0

    Get all spanners attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beam = beamtools.BeamSpanner(staff.leaves)
        >>> first_slur = spannertools.SlurSpanner(staff.leaves[:2])
        >>> second_slur = spannertools.SlurSpanner(staff.leaves[2:])
        >>> crescendo = spannertools.CrescendoSpanner(staff.leaves)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ \< (
            d'8 )
            e'8 (
            f'8 ] \! )
        }

    ::

        >>> result = spannertools.get_spanners_attached_to_component(staff.leaves[0])
        >>> for x in sorted(result):
        ...     x
        ...
        BeamSpanner(c'8, d'8, e'8, f'8)
        CrescendoSpanner(c'8, d'8, e'8, f'8)
        SlurSpanner(c'8, d'8)

    Get spanners of `klass` attached to `component`::

        >>> klass = beamtools.BeamSpanner
        >>> result = spannertools.get_spanners_attached_to_component(staff.leaves[0], klass)
        >>> for x in sorted(result):
        ...     x
        ...
        BeamSpanner(c'8, d'8, e'8, f'8)

    Get spanners of any `klass` attached to `component`::

        >>> klasses = (beamtools.BeamSpanner, spannertools.SlurSpanner)
        >>> result = spannertools.get_spanners_attached_to_component(staff.leaves[0], klasses)
        >>> for x in sorted(result):
        ...     x
        ...
        BeamSpanner(c'8, d'8, e'8, f'8)
        SlurSpanner(c'8, d'8)

    Return unordered set of zero or more spanners.

    .. versionchanged:: 2.0
        renamed ``spannertools.get_all_spanners_attached_to_component()`` to
        ``spannertools.get_spanners_attached_to_component()``.
    '''

    if klass is None:
        return component.spanners
    else:
        return set([x for x in component.spanners if isinstance(x, klass)])



from abjad import *


def test_marktools_detach_marks_attached_to_components_in_expr_01():

    staff = Staff("c'4 \staccato d' \marcato e' \staccato f' \marcato")
    assert len(marktools.get_marks_attached_to_components_in_expr(staff)) == 4

    marks = marktools.detach_marks_attached_to_components_in_expr(staff)
    assert marks == (
        marktools.Articulation('staccato'), 
        marktools.Articulation('marcato'), 
        marktools.Articulation('staccato'), 
        marktools.Articulation('marcato'))

    assert not len(marktools.get_marks_attached_to_components_in_expr(staff))



from abjad.tools import scoretools


def iterate_scores_in_expr(expr, reverse=False, start=0, stop=None):
    '''.. versionadded:: 2.10

    Iterate scores forward in `expr`::

        >>> score_1 = Score([Staff("c'8 d'8 e'8 f'8")])
        >>> score_2 = Score([Staff("c'1"), Staff("g'1")])
        >>> scores = [score_1, score_2]

    ::

        >>> for score in iterationtools.iterate_scores_in_expr(scores):
        ...   score
        Score<<1>>
        Score<<2>>

    Iterate scores backward in `expr`::

    ::

        >>> for score in iterationtools.iterate_scores_in_expr(scores, reverse=True):
        ...   score
        Score<<2>>
        Score<<1>>

    Ignore threads.

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=scoretools.Score, reverse=reverse, start=start, stop=stop)



# probably better with regular expression handling
def interval_string_to_pair_and_indicators(interval_string):
    r'''.. versionadded:: 1.0

    Change `interval_string` to pair, boolean start indicator and
    boolean stop indicator::

        >>> mathtools.interval_string_to_pair_and_indicators('[5, 8)')
        ((5, 8), False, True)

    Parse square brackets as closed interval bounds.

    Parse parentheses as open interval bounds.

    Return triple.
    '''

    assert isinstance(interval_string, str)
    spaceless_interval_string = interval_string.replace(' ', '')
    left, right = spaceless_interval_string.split(',')
    if left[0] == '(':
        is_left_open = True
    elif left[0] == '[':
        is_left_open = False
    else:
        raise ValueError(
            'can not initialize interval start from {!r}.'.format(interval_string))

    # probably attackable
    start = eval(left[1:])
    if right[-1] == ')':
        is_right_open = True
    elif right[-1] == ']':
        is_right_open = False
    else:
        raise ValueError(
            'can not initialize interval stop from {!r}.'.format(interval_string))

    # probably attackable
    stop = eval(right[:-1])
    pair = start, stop
    return pair, is_left_open, is_right_open



def list_known_grobs():
    '''List all LilyPond grobs recognized by ``LilyPondParser``:

    ::

        >>> for x in lilypondparsertools.list_known_grobs():
        ...     print x
        ...
        Accidental
        AccidentalCautionary
        AccidentalPlacement
        AccidentalSuggestion
        Ambitus
        AmbitusAccidental
        AmbitusLine
        AmbitusNoteHead
        Arpeggio
        BalloonTextItem
        BarLine
        BarNumber
        BassFigure
        BassFigureAlignment
        BassFigureAlignmentPositioning
        BassFigureBracket
        BassFigureContinuation
        BassFigureLine
        Beam
        BendAfter
        BreakAlignGroup
        BreakAlignment
        BreathingSign
        ChordName
        Clef
        ClusterSpanner
        ClusterSpannerBeacon
        CombineTextScript
        CueClef
        CueEndClef
        Custos
        DotColumn
        Dots
        DoublePercentRepeat
        DoublePercentRepeatCounter
        DoubleRepeatSlash
        DynamicLineSpanner
        DynamicText
        DynamicTextSpanner
        Episema
        Fingering
        FingeringColumn
        Flag
        FootnoteItem
        FootnoteSpanner
        FretBoard
        Glissando
        GraceSpacing
        GridLine
        GridPoint
        Hairpin
        HorizontalBracket
        InstrumentName
        InstrumentSwitch
        KeyCancellation
        KeySignature
        LaissezVibrerTie
        LaissezVibrerTieColumn
        LedgerLineSpanner
        LeftEdge
        LigatureBracket
        LyricExtender
        LyricHyphen
        LyricSpace
        LyricText
        MeasureCounter
        MeasureGrouping
        MelodyItem
        MensuralLigature
        MetronomeMark
        MultiMeasureRest
        MultiMeasureRestNumber
        MultiMeasureRestText
        NonMusicalPaperColumn
        NoteCollision
        NoteColumn
        NoteHead
        NoteName
        NoteSpacing
        OctavateEight
        OttavaBracket
        PaperColumn
        ParenthesesItem
        PercentRepeat
        PercentRepeatCounter
        PhrasingSlur
        PianoPedalBracket
        RehearsalMark
        RepeatSlash
        RepeatTie
        RepeatTieColumn
        Rest
        RestCollision
        Script
        ScriptColumn
        ScriptRow
        Slur
        SostenutoPedal
        SostenutoPedalLineSpanner
        SpacingSpanner
        SpanBar
        SpanBarStub
        StaffGrouper
        StaffSpacing
        StaffSymbol
        StanzaNumber
        Stem
        StemStub
        StemTremolo
        StringNumber
        StrokeFinger
        SustainPedal
        SustainPedalLineSpanner
        System
        SystemStartBar
        SystemStartBrace
        SystemStartBracket
        SystemStartSquare
        TabNoteHead
        TextScript
        TextSpanner
        Tie
        TieColumn
        TimeSignature
        TrillPitchAccidental
        TrillPitchGroup
        TrillPitchHead
        TrillSpanner
        TupletBracket
        TupletNumber
        UnaCordaPedal
        UnaCordaPedalLineSpanner
        VaticanaLigature
        VerticalAlignment
        VerticalAxisGroup
        VoiceFollower
        VoltaBracket
        VoltaBracketSpanner

    Return tuple.
    '''
    from abjad.ly import grob_interfaces

    return sorted(grob_interfaces.keys())



import copy
from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import sequencetools
from abjad.tools import pitchtools


def split_leaf_at_offsets(leaf, offsets, cyclic=False, 
    fracture_spanners=False, tie_split_notes=True, tie_split_rests=False):
    r'''.. versionadded:: 2.10

    Split `leaf` at `offsets`.

    Example 1. Split note once at `offsets` and tie split notes::

        >>> staff = Staff("c'1 ( d'1 )")

    ::

        >>> f(staff)
        \new Staff {
            c'1 (
            d'1 )
        }

    ::

        >>> leaftools.split_leaf_at_offsets(staff[0], [(3, 8)], 
        ...     tie_split_notes=True)
        [[Note("c'4.")], [Note("c'2"), Note("c'8")]]

    ::

        >>> f(staff)
        \new Staff {
            c'4. ( ~
            c'2 ~
            c'8
            d'1 )
        }

    Example 2. Split note cyclically at `offsets` and tie split notes::

        >>> staff = Staff("c'1 ( d'1 )")

    ::

        >>> f(staff)
        \new Staff {
            c'1 (
            d'1 )
        }

    ::

        >>> leaftools.split_leaf_at_offsets(staff[0], [(3, 8)], cyclic=True, 
        ...     tie_split_notes=True)
        [[Note("c'4.")], [Note("c'4.")], [Note("c'4")]]

    ::

        >>> f(staff)
        \new Staff {
            c'4. ( ~
            c'4. ~
            c'4
            d'1 )
        }

    Example 3. Split note once at `offsets` and do no tie split notes::

        >>> staff = Staff("c'1 ( d'1 )")

    ::

        >>> f(staff)
        \new Staff {
            c'1 (
            d'1 )
        }

    ::

        >>> leaftools.split_leaf_at_offsets(staff[0], [(3, 8)], 
        ...     tie_split_notes=False)
        [[Note("c'4.")], [Note("c'2"), Note("c'8")]]

    ::

        >>> f(staff)
        \new Staff {
            c'4. (
            c'2 ~
            c'8
            d'1 )
        }

    Example 4. Split note cyclically at `offsets` and do not tie split notes::

        >>> staff = Staff("c'1 ( d'1 )")

    ::

        >>> f(staff)
        \new Staff {
            c'1 (
            d'1 )
        }

    ::

        >>> leaftools.split_leaf_at_offsets(staff[0], [(3, 8)], cyclic=True, 
        ...     tie_split_notes=False)
        [[Note("c'4.")], [Note("c'4.")], [Note("c'4")]]

    ::

        >>> f(staff)
        \new Staff {
            c'4. (
            c'4.
            c'4
            d'1 )
        }

    Example 5. Split tupletted note once at `offsets` and tie split notes::

        >>> staff = Staff(r"\times 2/3 { c'2 ( d'2 e'2 ) }")

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'2 (
                d'2
                e'2 )
            }
        }

    ::

        >>> leaftools.split_leaf_at_offsets(staff.leaves[1], [(1, 6)], cyclic=False, 
        ...     tie_split_notes=True)
        [[Note("d'4")], [Note("d'4")]]

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'2 (
                d'4 ~
                d'4
                e'2 )
            }
        }

    .. note:: Add examples showing mark and context mark handling.

    Return list of shards.
    '''
    from abjad.tools import contexttools
    from abjad.tools import gracetools
    from abjad.tools import iterationtools
    from abjad.tools import leaftools
    from abjad.tools import marktools
    from abjad.tools import spannertools
    from abjad.tools import tietools
    
    assert isinstance(leaf, leaftools.Leaf) 
    offsets = [durationtools.Offset(offset) for offset in offsets]

    if cyclic:
        offsets = sequencetools.repeat_sequence_to_weight_exactly(offsets, leaf.prolated_duration)

    durations = [durationtools.Duration(offset) for offset in offsets]

    if sum(durations) < leaf.prolated_duration:
        last_duration = leaf.prolated_duration - sum(durations)
        durations.append(last_duration) 

    sequencetools.truncate_sequence_to_weight(durations, leaf.prolated_duration)

    result = []
    leaf_prolation = leaf.prolation
    leaf_copy = copy.deepcopy(leaf)
    for prolated_duration in durations:
        new_leaf = copy.deepcopy(leaf)
        preprolated_duration = prolated_duration / leaf_prolation
        shard = leaftools.set_preprolated_leaf_duration(new_leaf, preprolated_duration)
        shard = [x.parentage.root for x in shard]
        result.append(shard)

    flattened_result = sequencetools.flatten_sequence(result)
    if spannertools.get_spanners_attached_to_any_improper_parent_of_component(
        leaf, klass=tietools.TieSpanner):
        tietools.remove_tie_spanners_from_components_in_expr(flattened_result)
    componenttools.move_parentage_and_spanners_from_components_to_components([leaf], flattened_result)

    if fracture_spanners:
        first_shard = result[0]
        spannertools.fracture_spanners_attached_to_component(first_shard[-1], direction=Right) 
        last_shard = result[-1]
        spannertools.fracture_spanners_attached_to_component(last_shard[0], direction=Left) 
        for middle_shard in result[1:-1]:
            spannertools.fracture_spanners_attached_to_component(middle_shard[0], direction=Left) 
            spannertools.fracture_spanners_attached_to_component(middle_shard[-1], direction=Right) 
    
    # adjust first leaf
    first_leaf = flattened_result[0]
    gracetools.detach_grace_containers_attached_to_leaf(leaf, kind='after')

    # adjust any middle leaves
    for middle_leaf in flattened_result[1:-1]:
        gracetools.detach_grace_containers_attached_to_leaf(middle_leaf, kind='grace')
        gracetools.detach_grace_containers_attached_to_leaf(leaf, kind='after')
        marktools.detach_marks_attached_to_component(middle_leaf)
        contexttools.detach_context_marks_attached_to_component(middle_leaf)

    # adjust last leaf
    last_leaf = flattened_result[-1]
    gracetools.detach_grace_containers_attached_to_leaf(last_leaf, kind='grace')
    marktools.detach_marks_attached_to_component(last_leaf)
    contexttools.detach_context_marks_attached_to_component(last_leaf)

    # tie split notes, rests and chords as specified
    if  (pitchtools.is_pitch_carrier(leaf) and tie_split_notes) or \
        (not pitchtools.is_pitch_carrier(leaf) and tie_split_rests):
        flattened_result_leaves = iterationtools.iterate_leaves_in_expr(flattened_result)
        # TODO: maybe generalize tietools.apply_tie_spanner_to_leaf_pair()
        #       to tietools.apply_tie_spanner_to_leaves().
        for leaf_pair in sequencetools.iterate_sequence_pairwise_strict(flattened_result_leaves):
            tietools.apply_tie_spanner_to_leaf_pair(*leaf_pair)

    # return result
    return result



from abjad.tools import componenttools
from abjad.tools import containertools
from abjad.tools import durationtools
from abjad.tools import mathtools


def scale_measure_and_adjust_time_signature(measure, multiplier=1):
    r'''.. versionadded:: 2.0

    Scale `measure` by `multiplier` and adjust time signature::

        >>> t = Measure((3, 8), "c'8 d'8 e'8")
        >>> measuretools.scale_measure_and_adjust_time_signature(t, Duration(2, 3))
        Measure(3/12, [c'8, d'8, e'8])

    ::

        >>> f(t)
        {
        \time 3/12
        \scaleDurations #'(2 . 3) {
            c'8
            d'8
            e'8
        }
        }

    Return `measure`.
    '''
    from abjad.tools import contexttools

    if multiplier == 0:
        raise ZeroDivisionError

    old_time_signature = contexttools.get_effective_time_signature(measure)
    old_pair = (old_time_signature.numerator, old_time_signature.denominator)
    old_multiplier = old_time_signature.implied_prolation
    old_multiplier_pair = (old_multiplier.numerator, old_multiplier.denominator)

    multiplied_pair = mathtools.NonreducedFraction(old_multiplier_pair)
    multiplied_pair = multiplied_pair.multiply_without_reducing(multiplier)
    multiplied_pair = multiplied_pair.pair
    reduced_pair = mathtools.NonreducedFraction(old_multiplier_pair)
    reduced_pair = reduced_pair.multiply_with_cross_cancelation(multiplier)
    reduced_pair = reduced_pair.pair

    if reduced_pair != multiplied_pair:
        new_pair = mathtools.NonreducedFraction(old_pair)
        new_pair = new_pair.multiply_with_numerator_preservation(multiplier)
        new_time_signature = contexttools.TimeSignatureMark(new_pair)
        contexttools.detach_time_signature_marks_attached_to_component(measure)
        new_time_signature.attach(measure)
        remaining_multiplier = durationtools.Duration(reduced_pair)
        if remaining_multiplier != durationtools.Duration(1):
            containertools.scale_contents_of_container(measure, remaining_multiplier)
    elif componenttools.all_are_components_scalable_by_multiplier(measure[:], multiplier):
        containertools.scale_contents_of_container(measure, multiplier)
        if old_time_signature.has_non_power_of_two_denominator or not mathtools.is_nonnegative_integer_power_of_two(multiplier):
            new_pair = mathtools.NonreducedFraction(old_pair)
            new_pair = new_pair.multiply_with_cross_cancelation(multiplier)
            new_pair = new_pair.pair
        # multiplier is a negative power of two, like 1/2, 1/4, etc.
        elif multiplier < durationtools.Duration(0):
            new_pair = durationtools.multiply_duration_pair(old_pair, multiplier)
        # multiplier is a nonnegative power of two, like 0, 1, 2, 4, etc.
        elif durationtools.Duration(0) < multiplier:
            new_pair = mathtools.NonreducedFraction(old_pair)
            new_pair = new_pair.multiply_with_numerator_preservation(multiplier)
        elif multiplier == durationtools.Duration(0):
            raise ZeroDivisionError
        new_time_signature = contexttools.TimeSignatureMark(new_pair)
        contexttools.detach_time_signature_marks_attached_to_component(measure)
        new_time_signature.attach(measure)
    else:
        new_pair = mathtools.NonreducedFraction(old_pair)
        new_pair = new_pair.multiply_with_numerator_preservation(multiplier)
        new_time_signature = contexttools.TimeSignatureMark(new_pair)
        contexttools.detach_time_signature_marks_attached_to_component(measure)
        new_time_signature.attach(measure)
        remaining_multiplier = multiplier / new_time_signature.implied_prolation
        if remaining_multiplier != durationtools.Duration(1):
            containertools.scale_contents_of_container(measure, remaining_multiplier)
    return measure



def is_pair(expr):
    r'''.. versionadded:: 2.9

    True when `expr` is a tuple of length ``2``::

        >>> sequencetools.is_pair((19, 20))
        True

    Otherwise false::

        >>> sequencetools.is_pair((19, 20, 21))
        False

    Return boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == 2



def timespan_2_trisects_timespan_1(timespan_1=None, timespan_2=None, hold=False):
    r'''.. versionadded:: 2.11

    Make time relation indicating that `timespan_2` trisects `timespan_1`::

        >>> timerelationtools.timespan_2_trisects_timespan_1()
        TimespanTimespanTimeRelation('timespan_1.start < timespan_2.start and timespan_2.stop < timespan_1.stop')

    Return boolean or time relation.
    '''
    from abjad.tools import timerelationtools

    time_relation = timerelationtools.TimespanTimespanTimeRelation(
        'timespan_1.start < timespan_2.start and timespan_2.stop < timespan_1.stop',
        timespan_1=timespan_1,
        timespan_2=timespan_2)

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



import abc

from abjad.tools.datastructuretools import ImmutableDictionary


class ObjectVector(ImmutableDictionary):
    '''.. versionadded:: 2.0

    Music theoretic vector base class.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass



from abjad.tools import mathtools
from abjad.tools import sequencetools


# TODO: Implement in-place containertools.partition_components_by_counts()
#       that doesn't climb to governor.
def copy_and_partition_governed_component_subtree_by_leaf_counts(container, leaf_counts):
    r'''.. versionadded:: 1.1

    Copy `container` and partition copy according to `leaf_counts`::

        >>> voice = Voice(r"\times 2/3 { c'8 d'8 e'8 } \times 2/3 { f'8 g'8 a'8 }")
        >>> beamtools.BeamSpanner(voice[0].leaves)
        BeamSpanner(c'8, d'8, e'8)
        >>> beamtools.BeamSpanner(voice[1].leaves)
        BeamSpanner(f'8, g'8, a'8)

    ::

        >>> f(voice)
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8 ]
            }
            \times 2/3 {
                f'8 [
                g'8
                a'8 ]
            }
        }

    ::

        >>> result = componenttools.copy_and_partition_governed_component_subtree_by_leaf_counts(
        ... voice, [1, 2, 3])

    ::

        >>> first, second, third = result

    ::

        >>> f(first) # doctest: +SKIP
        \new Voice {
            \times 2/3 {
                c'8 [ ]
            }
        }

    ::

        >>> f(second) # doctest: +SKIP
        \new Voice {
            \times 2/3 {
                d'8 [
                e'8 ]
            }
        }

    ::

        >>> f(third) # doctest: +SKIP
        \new Voice {
            \times 2/3 {
                f'8 [
                g'8
                a'8 ]
            }
        }

    Set `leaf_counts` to an iterable of zero or more positive integers.

    Return a list of parts equal in length to that of `leaf_counts`.

    .. versionchanged:: 2.0
        renamed ``clonewp.by_leaf_counts_with_parentage()`` to
        ``componenttools.copy_and_partition_governed_component_subtree_by_leaf_counts()``.
    '''
    from abjad.tools import componenttools
    from abjad.tools import containertools

    assert isinstance(container, containertools.Container)
    assert all([isinstance(x, int) for x in leaf_counts])

    result = []
    sums = mathtools.cumulative_sums_zero(leaf_counts)
    for start, stop in sequencetools.iterate_sequence_pairwise_strict(sums):
        result.append(componenttools.copy_governed_component_subtree_by_leaf_range(container, start, stop))
    return result



def get_component_in_expr_with_name(expr, name):
    r""".. versionadded:: 2.10

    Get component in `expr` with `name`:

    ::

        >>> voice_1 = Voice("c'4 d'4 e'4 f'4")
        >>> voice_1.name = 'Top Voice'
        >>> voice_2 = Voice(r'\clef "bass" c4 d4 e4 f4')
        >>> voice_2.name = 'Bottom Voice'
        >>> staff = Staff([voice_1, voice_2])
        >>> f(staff)
        \new Staff {
            \context Voice = "Top Voice" {
                c'4
                d'4
                e'4
                f'4
            }
            \context Voice = "Bottom Voice" {
                \clef "bass"
                c4
                d4
                e4
                f4
            }
        }

    ::

        >>> voice = componenttools.get_component_in_expr_with_name(staff, 'Top Voice')
        >>> f(voice)
        \context Voice = "Top Voice" {
            c'4
            d'4
            e'4
            f'4
        }    

    Return one component.

    Raise missing component error when no named component is found.

    Raise extra component error when more than one component with `name` is found.
    """
    from abjad.tools import componenttools

    components = componenttools.get_components_in_expr_with_name(expr, name)
    if not components:
        raise MissingNamedComponentError
    elif 1 < len(components):
        raise ExtraNamedComponentError
    else:
        return components[0]




from abjad.tools.spannertools.Spanner import Spanner


class HorizontalBracketSpanner(Spanner):
    r'''.. versionadded:: 2.4

    Abjad horizontal bracket spanner::

        >>> voice = Voice("c'4 d'4 e'4 f'4")
        >>> voice.engraver_consists.append('Horizontal_bracket_engraver')

    ::

        >>> horizontal_bracket_spanner = spannertools.HorizontalBracketSpanner(voice[:])

    ::

        >>> horizontal_bracket_spanner
        HorizontalBracketSpanner(c'4, d'4, e'4, f'4)

    ::

        >>> f(voice)
        \new Voice \with {
            \consists Horizontal_bracket_engraver
        } {
            c'4 \startGroup
            d'4
            e'4
            f'4 \stopGroup
        }

    ::

        >>> show(voice) # doctest: +SKIP

    Return horizontal bracket spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None):
        Spanner.__init__(self, components)

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\startGroup')
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopGroup')
        return result



from abjad.tools import chordtools
from abjad.tools import durationtools
from abjad.tools import notetools


def is_beamable_component(expr):
    '''.. versionadded:: 1.1

    True when `expr` is a beamable component. Otherwise false::

        >>> beamtools.is_beamable_component(Note(13, (1, 16)))
        True

    Return boolean.

    .. versionchanged:: 2.9
        renamed ``componenttools.is_beamable_component()`` to
        ``beamtools.is_beamable_component()``.
    '''

    if isinstance(expr, (notetools.Note, chordtools.Chord)):
        if 0 < expr.written_duration.flag_count:
            return True
    return False



def timespan_2_overlaps_all_of_timespan_1(timespan_1=None, timespan_2=None, hold=False):
    r'''.. versionadded:: 2.11

    Make time relation template indicating that expression overlaps all of timespan::

        >>> timerelationtools.timespan_2_overlaps_all_of_timespan_1()
        TimespanTimespanTimeRelation('timespan_2.start < timespan_1.start < timespan_1.stop < timespan_2.stop')

    Return boolean or time relation.
    '''
    from abjad.tools import timerelationtools

    time_relation = timerelationtools.TimespanTimespanTimeRelation(
        'timespan_2.start < timespan_1.start < timespan_1.stop < timespan_2.stop',
        timespan_1=timespan_1, 
        timespan_2=timespan_2)
    
    if time_relation.is_fully_loaded and not hold: 
        return time_relation()
    else:
        return time_relation



from abjad.tools import durationtools


def update_offset_values_of_component(component):
    r'''.. versionadded:: 2.9

    Update prolated offset values of `component`.
    '''
    from abjad.tools import componenttools
    prev = componenttools.get_nth_component_in_time_order_from_component(component, -1)
    if prev is not None:
        component._start_offset = prev._stop_offset
    else:
        component._start_offset = durationtools.Offset(0)
    component._stop_offset = component._start_offset + component.prolated_duration



def make_piano_sketch_score_from_leaves(leaves, lowest_treble_pitch=None):
    r'''.. versionadded:: 2.0

    Make piano sketch score from `leaves`::

        >>> notes = notetools.make_notes([-12, -10, -8, -7, -5, 0, 2, 4, 5, 7], [(1, 4)])
        >>> score, treble_staff, bass_staff = scoretools.make_piano_sketch_score_from_leaves(notes)

    ::

        >>> f(score)
        \new Score \with {
            \override BarLine #'stencil = ##f
            \override BarNumber #'transparent = ##t
            \override SpanBar #'stencil = ##f
            \override TimeSignature #'stencil = ##f
        } <<
            \new PianoStaff <<
                \context Staff = "treble" {
                    \clef "treble"
                    #(set-accidental-style 'forget)
                    r4
                    r4
                    r4
                    r4
                    r4
                    c'4
                    d'4
                    e'4
                    f'4
                    g'4
                }
                \context Staff = "bass" {
                    \clef "bass"
                    #(set-accidental-style 'forget)
                    c4
                    d4
                    e4
                    f4
                    g4
                    r4
                    r4
                    r4
                    r4
                    r4
                }
            >>
        >>

    ::

        >>> show(score) # doctest: +SKIP

    When ``lowest_treble_pitch=None`` set to B3.

    Make time signatures and bar numbers transparent.

    Do not print bar lines or span bars.

    Set all staff accidental styles to forget.

    Return score, treble staff, bass staff.
    '''
    from abjad.tools import contexttools
    from abjad.tools import lilypondfiletools
    from abjad.tools import markuptools
    from abjad.tools import pitchtools
    from abjad.tools import scoretools

    if lowest_treble_pitch is None:
        lowest_treble_pitch = pitchtools.NamedChromaticPitch('b')

    # make and configure score
    score, treble_staff, bass_staff = scoretools.make_piano_score_from_leaves(leaves, lowest_treble_pitch)
    score.override.time_signature.stencil = False
    score.override.bar_number.transparent = True
    score.override.bar_line.stencil = False
    score.override.span_bar.stencil = False
    contexttools.set_accidental_style_on_sequential_contexts_in_expr(score, 'forget')

    # make and configure lily file
    lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
    lilypond_file.layout_block.indent = 0
    lilypond_file.paper_block.tagline = markuptools.Markup('')

    # return score, treble staff, bass staff
    return score, treble_staff, bass_staff



from abjad.tools import containertools
from abjad.tools import iterationtools
from abjad.tools.wellformednesstools.Check import Check


class EmptyContainerCheck(Check):

    runtime = 'composition'

    def _run(self, expr):
        violators = []
        bad, total = 0, 0
        for t in iterationtools.iterate_components_in_expr(expr, containertools.Container):
            if len(t) == 0:
                violators.append(t)
                bad += 1
            total += 1
        return violators, total



from abjad.tools import iotools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
from abjad.tools.developerscripttools.CleanScript import CleanScript
import argparse
import os


class AbjGrepScript(DirectoryScript):
    '''Run `grep` against a path, ignoring `svn` and docs-related files:

    ::

        bash$ ajv grep -h
        usage: abj-grep [-h] [--version] [-W] [-P PATH | -X | -M | -T | -R] pattern

        grep PATTERN in PATH

        positional arguments:
          pattern               pattern to search for

        optional arguments:
          -h, --help            show this help message and exit
          --version             show program's version number and exit
          -W, --whole-words-only
                                match only whole words, similar to grep's "-w" flag
          -P PATH, --path PATH  grep PATH
          -X, --abjad.tools    grep Abjad abjad.tools directory
          -M, --mainline        grep Abjad mainline directory
          -T, --tools           grep Abjad mainline tools directory
          -R, --root            grep Abjad root directory

        If no PATH flag is specified, the current directory will be searched.
            
    Return `AbjGrepScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'grep'

    @property
    def long_description(self):
        return '''\
If no PATH flag is specified, the current directory will be searched.
    '''

    @property
    def scripting_group(self):
        return None

    @property
    def short_description(self):
        return 'grep PATTERN in PATH'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):

        iotools.clear_terminal()
        if args.whole_words_only:
            whole_words_only = '-w'
        else:
            whole_words_only = ''
        command = r'grep {} -Irn {!r} {} | grep -v svn-base | grep -v svn\/ | grep -v docs'.format(
            whole_words_only, args.pattern, os.path.relpath(args.path))
        iotools.spawn_subprocess(command)

    def setup_argument_parser(self, parser):

        from abjad import ABJCFG

        parser.add_argument('pattern',
            help='pattern to search for'
            )

        parser.add_argument('-W', '--whole-words-only',
            action='store_true',
            help='''match only whole words, similar to grep's "-w" flag''',
            )

        group = parser.add_mutually_exclusive_group()

        group.add_argument('-P', '--path',
            dest='path',
            help='grep PATH',
            type=self._validate_path,
            )

        group.add_argument('-X', '--experimental',
            action='store_const',
            const=ABJCFG.ABJAD_EXPERIMENTAL_PATH,
            dest='path',
            help='grep Abjad abjad.tools directory',
            )

        group.add_argument('-M', '--mainline',
            action='store_const',
            const=ABJCFG.ABJAD_PATH,
            dest='path',
            help='grep Abjad mainline directory',
            )

        group.add_argument('-T', '--tools',
            action='store_const',
            const=os.path.join(ABJCFG.ABJAD_PATH, 'tools'),
            dest='path',
            help='grep Abjad mainline tools directory',
            )

        group.add_argument('-R', '--root',
            action='store_const',
            const=ABJCFG.ABJAD_ROOT_PATH,
            dest='path',
            help='grep Abjad root directory',
            )

        parser.set_defaults(path=os.path.abspath(os.path.curdir))



def set_written_pitch_of_pitched_components_in_expr(expr, written_pitch=0):
    r'''.. versionadded:: 2.9

    Set written pitch of pitched components in `expr` to `written_pitch`::

        >>> staff = Staff("c' d' e' f'")

    ::

        >>> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    ::

        >>> pitchtools.set_written_pitch_of_pitched_components_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            c'4
            c'4
            c'4
            c'4
        }

    Use as a way of neutralizing pitch information in an arbitrary piece of score.

    Return none.
    '''
    from abjad.tools import chordtools
    from abjad.tools import iterationtools
    from abjad.tools import notetools

    for leaf in iterationtools.iterate_leaves_in_expr(expr):
        if isinstance(leaf, notetools.Note):
            leaf.written_pitch = written_pitch
        elif isinstance(leaf, chordtools.Chord):
            leaf.written_pitches = [written_pitch]



def set_always_format_time_signature_of_measures_in_expr(expr, value=True):
    '''.. versionadded:: 2.9

    Set `always_format_time_signature` of measures in `expr` to boolean `value`.

    Return none.
    '''
    from abjad.tools import iterationtools

    for measure in iterationtools.iterate_measures_in_expr(expr):
        measure.always_format_time_signature = value



from abjad import *


def test_MarkupInventory_storage_format_01():

    inventory = markuptools.MarkupInventory(['foo', 'bar'])

    r'''
    markuptools.MarkupInventory([
        markuptools.Markup((
            'foo',
            )),
        markuptools.Markup((
            'bar',
            ))
        ])
    '''

    assert inventory.storage_format == "markuptools.MarkupInventory([\n\tmarkuptools.Markup((\n\t\t'foo',\n\t\t)),\n\tmarkuptools.Markup((\n\t\t'bar',\n\t\t))\n\t])"




def partition_components_by_durations_not_less_than(components, durations,     
    cyclic=False, in_seconds=False, overhang=False):
    r'''.. versionadded:: 1.1

    Partition `components` by `durations`.

    Example 1. Partition `components` cyclically by prolated `durations`. Keep overhang::

        >>> string = "abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 || 2/8 b'8 c''8 |"
        >>> staff = Staff(string)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }

    ::

        >>> parts = componenttools.partition_components_by_durations_not_less_than(
        ...     staff.leaves, [Duration(3, 16), Duration(1, 16)], cyclic=True, overhang=True)

    ::

        >>> for part in parts:
        ...     part
        ...     
        [Note("c'8"), Note("d'8")] 
        [Note("e'8")] 
        [Note("f'8"), Note("g'8")]
        [Note("a'8")] 
        [Note("b'8"), Note("c''8")]

    Return list of lists.

    Function works not just on components but on any durated objects including spanners.
    '''
    from abjad.tools.componenttools._partition_components_by_durations import _partition_components_by_durations

    if in_seconds:
        duration_type = 'seconds'
    else:
        duration_type = 'prolated'

    return _partition_components_by_durations(duration_type, components, durations,
        fill='greater', cyclic=cyclic, overhang=overhang)



from abjad import *


def test_skiptools_make_repeated_skips_from_time_signature_01():
    '''Make repeated skips from integer pair.
    '''

    skips = skiptools.make_repeated_skips_from_time_signature((5, 32))
    staff = Staff(skips)

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == '\\new Staff {\n\ts32\n\ts32\n\ts32\n\ts32\n\ts32\n}'


def test_skiptools_make_repeated_skips_from_time_signature_02():
    '''Make repeated skips from time signature.
    '''

    time_signature = contexttools.TimeSignatureMark((5, 32))
    skips = skiptools.make_repeated_skips_from_time_signature(time_signature)
    staff = Staff(skips)

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == '\\new Staff {\n\ts32\n\ts32\n\ts32\n\ts32\n\ts32\n}'



from abjad.tools import durationtools


def update_offset_values_of_component_in_seconds(component):
    r'''.. versionadded:: 2.9

    Update offset values of `component` in seconds.
    '''
    from abjad.tools import componenttools
    try:
        current_duration_in_seconds = component.duration_in_seconds
        prev = componenttools.get_nth_component_in_time_order_from_component(component, -1)
        if prev is not None:
            component._start_offset_in_seconds = prev.stop_offset_in_seconds
        else:
            component._start_offset_in_seconds = durationtools.Offset(0)
        # this one case is possible for containers only
        if component._start_offset_in_seconds is None:
            raise MissingTempoError
        component._stop_offset_in_seconds = component._start_offset_in_seconds + current_duration_in_seconds
    except MissingTempoError:
        pass



import os


def get_text_editor():
    '''.. versionadded:: 2.2

    Get OS-appropriate text editor.
    '''
    from abjad import ABJCFG
    from abjad.tools import configurationtools

    text_editor = ABJCFG['text_editor']

    if text_editor is not None:
        return text_editor
    elif os.name == 'posix':
        return 'vim'
    else:
        return 'edit'



from abjad.tools import componenttools
from abjad.tools import spannertools


def is_component_with_tie_spanner_attached(expr):
    r'''.. versionadded:: 2.0

    True when `expr` is component with tie spanner attached::

        >>> staff = Staff("c'8 ~ c' ~ c' ~ c'")

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'8 ~
            c'8 ~
            c'8
        }

    ::

        >>> tietools.is_component_with_tie_spanner_attached(staff[1])
        True

    Otherwise false::

        >>> tietools.is_component_with_tie_spanner_attached(staff)
        False

    Return boolean.
    '''
    from abjad.tools import tietools

    if not isinstance(expr, componenttools.Component):
        return False

    return bool(spannertools.get_spanners_attached_to_component(expr, tietools.TieSpanner))



from abjad.tools import iterationtools
from abjad.tools import spannertools
from abjad.tools.wellformednesstools.Check import Check


class OverlappingGlissandoCheck(Check):
    '''Glissandi must not overlap.
    Dove-tailed glissandi are OK.
    '''

    def _run(self, expr):
        violators = []
        for leaf in iterationtools.iterate_leaves_in_expr(expr):
            glissandi = spannertools.get_spanners_attached_to_component(
                leaf, spannertools.GlissandoSpanner)
            if 1 < len(glissandi):
                if len(glissandi) == 2:
                    common_leaves = set(glissandi[0].leaves) & \
                        set(glissandi[1].leaves)
                    if len(common_leaves) == 1:
                        x = list(common_leaves)[0]
                        if (glissandi[0]._is_my_first_leaf(x) and
                            glissandi[1]._is_my_last_leaf(x)) or \
                            (glissandi[1]._is_my_first_leaf(x) and
                            glissandi[0]._is_my_last_leaf(x)):
                            break

                for glissando in glissandi:
                    if glissando not in violators:
                        violators.append(glissando)
        total = spannertools.get_spanners_attached_to_any_improper_child_of_component(
            expr, spannertools.GlissandoSpanner)
        return violators, len(total)



import os
import subprocess


def plot(expr, image_format='png', width=640, height=320):
    '''Plot `expr` with gnuplot, and open resulting image in the default image viewer.

    Return None.
    '''

    from abjad import ABJCFG
    from abjad.tools import iotools
    from abjad.tools.iotools._open_file import _open_file
    from abjad.tools.iotools._verify_output_directory import _verify_output_directory

    assert image_format in ('pdf', 'png')
    assert isinstance(width, int) and 0 < width
    assert isinstance(height, int) and 0 < height

    gnuplot_format = expr.gnuplot_format

    current_directory = os.path.abspath('.')
    ABJADOUTPUT = ABJCFG['abjad_output']
    _verify_output_directory(ABJADOUTPUT)
    txt_path = os.path.join(ABJADOUTPUT, iotools.get_next_output_file_name(file_extension='txt'))
    img_path = os.path.join(ABJADOUTPUT, txt_path.replace('txt', image_format))

    if image_format == 'png':
        image_format = 'pngcairo'

    gnuplot_format = gnuplot_format.format(
        filename=img_path,
        image_format=image_format,
        height=height,
        width=width
        )

    with open(txt_path, 'w') as f:
        f.write(gnuplot_format)

    command = 'gnuplot {}'.format(txt_path)
    subprocess.call(command, shell=True)

    pdf_viewer = ABJCFG['pdf_viewer']
    ABJADOUTPUT = ABJCFG['abjad_output']
    _open_file(img_path, pdf_viewer)





from abjad import *


def test_SchemePair_lilypond_format_01():
    '''Right-hand side string forces quotes.
    '''

    scheme_pair = schemetools.SchemePair('font-name', 'Times')
    assert scheme_pair.lilypond_format == '#\'(font-name . "Times")'


def test_SchemePair_lilypond_format_02():
    '''Right-hand side nonstring does not force quotes.
    '''

    scheme_pair = schemetools.SchemePair('spacing', 4)
    assert scheme_pair.lilypond_format == "#'(spacing . 4)"



import fractions


def calculate_depth_density_of_intervals(intervals):
    '''Return a Fraction, of the duration of each interval in the
    depth tree of `intervals`, multiplied by the depth at that interval,
    divided by the overall duration of `intervals`.

    The depth density of a single interval is 1 ::

        >>> from abjad.tools.timeintervaltools import TimeInterval
        >>> from abjad.tools.timeintervaltools import TimeIntervalTree

    ::

        >>> a = TimeInterval(0, 1)
        >>> b = TimeInterval(0, 1)
        >>> c = TimeInterval(Fraction(1, 2), 1)
        >>> timeintervaltools.calculate_depth_density_of_intervals(a)
        Duration(1, 1)
        >>> timeintervaltools.calculate_depth_density_of_intervals([a, b])
        Duration(2, 1)
        >>> timeintervaltools.calculate_depth_density_of_intervals([a, c])
        Duration(3, 2)
        >>> timeintervaltools.calculate_depth_density_of_intervals([a, b, c])
        Duration(5, 2)

    Return fraction.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return fractions.Fraction(0)

    return fractions.Fraction(sum([x.duration for x in tree])) / tree.duration



from abjad.tools.containertools.Container import Container


class Cluster(Container):
    '''.. versionadded:: 1.1

    Abjad model of a tone cluster container::

        >>> cluster = containertools.Cluster("c'8 <d' g'>8 b'8")

    ::

        >>> cluster
        Cluster(c'8, <d' g'>8, b'8)

    ::

        >>> f(cluster)
        \makeClusters {
            c'8
            <d' g'>8
            b'8
        }

    ::

        >>> show(cluster) # doctest: +SKIP

    Return cluster object.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, music=None, **kwargs):
        Container.__init__(self, music)
        self.is_parallel = False
        self._initialize_keyword_values(**kwargs)

    ### SPECIAL METHODS ###

    def __repr__(self):
        return '%s(%s)' % (type(self).__name__, self._summary)

    ### PRIVATE METHODS ###

    def _format_open_brackets_slot(self, format_contributions):
        result = []
        contributor = ('self_brackets', 'open')
        if self.is_parallel:
            brackets_open = ['<<']
        else:
            brackets_open = ['{']
        contributions = [r'\makeClusters %s' % brackets_open[0]]
        result.append([contributor, contributions])
        return tuple(result)
    
    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def lilypond_format(self):
        self._update_marks_of_entire_score_tree_if_necessary()
        return self._format_component()



def get_proper_contents_of_component(component):
    r'''.. versionadded:: 2.9

    Get proper contents of `component`::

        >>> staff = Staff("c' d' e' f'")

    ::

        >>> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    ::

        >>> componenttools.get_proper_contents_of_component(staff)
        [Note("c'4"), Note("d'4"), Note("e'4"), Note("f'4")]

    The function works on leaves::

        >>> componenttools.get_proper_contents_of_component(staff[0])
        []

    Return list of the proper contents of component.
    '''

    return list(getattr(component, 'music', []))



def get_composite_offset_series_from_leaves_in_expr(expr):
    r'''.. versionadded:: 2.0

    Get composite offset series from leaves in `expr`::

        >>> staff_1 = Staff(r"\times 4/3 { c'8 d'8 e'8 }")
        >>> staff_2 = Staff("f'8 g'8 a'8 b'8")
        >>> score = Score([staff_1, staff_2])

    ::

        >>> f(score)
            \new Score <<
                \new Staff {
                    \fraction \times 4/3 {
                        c'8
                        d'8
                        e'8
                    }
                }
                \new Staff {
                    f'8
                    g'8
                    a'8
                    b'8
                }
            >>

    ::

        >>> for offset in leaftools.get_composite_offset_series_from_leaves_in_expr(score):
        ...     offset
        ...
        Offset(0, 1)
        Offset(1, 8)
        Offset(1, 6)
        Offset(1, 4)
        Offset(1, 3)
        Offset(3, 8)
        Offset(1, 2)

    Equal to list of unique start and stop offsets of leaves in `expr`.

    Return list of fractions.
    '''
    from abjad.tools import iterationtools

    offsets = []

    for leaf in iterationtools.iterate_leaves_in_expr(expr):
        start_offset = leaf.start_offset
        if start_offset not in offsets:
            offsets.append(start_offset)
        stop_offset = leaf.stop_offset
        if stop_offset not in offsets:
            offsets.append(stop_offset)

    offsets.sort()

    return offsets



from abjad.tools import durationtools


def get_first_element_starting_strictly_before_offset(container, prolated_offset):
    '''.. versionadded:: 2.0

    Get first `container` element starting strictly before `prolated_offset`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> containertools.get_first_element_starting_strictly_before_offset(staff, Duration(1, 8))
        Note("c'8")

    Return component.

    Return none when `container` element starts stirctly before `prolated_offset`.

    .. versionchanged:: 2.0
        renamed ``containertools.get_rightmost_element_starting_before_prolated_offset()`` to
        ``containertools.get_first_element_starting_strictly_before_offset()``.
    '''

    prolated_offset = durationtools.Duration(prolated_offset)

    for element in reversed(container):
        if element.start_offset < prolated_offset:
            return element



from abjad.tools.spannertools.Spanner import Spanner


# TODO: remove and just use Spanner instead with overrides
class TextScriptSpanner(Spanner):
    r'''.. versionadded:: 2.0

    Abjad text script spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spanner = spannertools.TextScriptSpanner(staff[:])
        >>> spanner.override.text_script.color = 'red'
        >>> markuptools.Markup(r'\italic { espressivo }', Up)(staff[1])
        Markup((MarkupCommand('italic', ['espressivo']),), direction=Up)(d'8)

    ::

        >>> f(staff)
        \new Staff {
            \override TextScript #'color = #red
            c'8
            d'8 ^ \markup { \italic { espressivo } }
            e'8
            f'8
            \revert TextScript #'color
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Override LilyPond TextScript grob.

    Return text script spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None):
        Spanner.__init__(self, components)

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass



from abjad.tools import componenttools
from abjad.tools import leaftools


def insert_component(container, i, component, fracture_spanners=False):
    r'''.. versionadded:: 2.10

    Insert `component` into `container` at index `i`.

    Example 1.  Insert `component` into `container` at index `i`.
    Do not fracture crossing spanners::

        >>> staff = Staff("c'8 [ d'8 e'8 f'8 ]")

    ::

        >>> f(staff)
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> containertools.insert_component(staff, 1, Note("cs'8"), fracture_spanners=False)
        Staff{5}

    ::

        >>> f(staff)
        \new Staff {
            c'8 [
            cs'8
            d'8
            e'8
            f'8 ]
        }

    Example 2.  Insert `component` into `container` at index `i`.
    Fracture crossing spanners.

        >>> staff = Staff("c'8 [ d'8 e'8 f'8 ]")

    ::

        >>> f(staff)
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> parts = containertools.insert_component(staff, 1, Rest('r8'), fracture_spanners=True)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ]
            r8
            d'8 [
            e'8
            f'8 ]
        }

    Return `container` or list of fractured spanners.
    '''
    from abjad.tools import containertools
    from abjad.tools import spannertools

    # check input
    assert isinstance(container, containertools.Container)
    assert isinstance(component, componenttools.Component)
    assert isinstance(i, int)

    if not fracture_spanners:

        # insert component into container at index i
        # to avoid pychecker slice assignment error
        #container[i:i] = [component]
        container.__setitem__(slice(i, i), [component])

        # return container
        return container

    result = []
    component._switch(container)
    container._music.insert(i, component)

    previous_leaf = leaftools.get_nth_leaf_in_thread_from_leaf(component, -1)
    if previous_leaf:
        result.extend(spannertools.fracture_spanners_attached_to_component(previous_leaf, direction=Right))

    next_leaf = leaftools.get_nth_leaf_in_thread_from_leaf(component, 1)
    if next_leaf:
        result.extend(spannertools.fracture_spanners_attached_to_component(next_leaf, direction=Left))

    return result



def calculate_min_mean_and_max_depth_of_intervals(intervals):
    '''Return a 3-tuple of the minimum, mean and maximum depth of `intervals`.
    If `intervals` is empty, return None.  "Mean" in this case is a weighted mean,
    where the durations of the intervals in depth tree of `intervals` are the weights
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return None

    depth_tree = timeintervaltools.compute_depth_of_intervals(tree)
    depths = [x['depth'] for x in depth_tree]
    mean = timeintervaltools.calculate_depth_density_of_intervals(tree)
    return min(depths), mean, max(depths)



def list_numbered_chromatic_pitch_classes_in_expr(expr):
    '''.. versionadded:: 2.0

    List numbered chromatic pitch-classes in `expr`::

        >>> chord = Chord("<cs'' d'' ef''>4")

    ::

        >>> for x in pitchtools.list_numbered_chromatic_pitch_classes_in_expr(chord):
        ...     x
        ...
        NumberedChromaticPitchClass(1)
        NumberedChromaticPitchClass(2)
        NumberedChromaticPitchClass(3)

    Works with notes, chords, defective chords.

    Return tuple or zero or more numbered chromatic pitch-classes.

    .. versionchanged:: 2.0
        renamed ``pitchtools.list_numeric_chromatic_pitch_classes_in_expr()`` to
        ``pitchtools.list_numbered_chromatic_pitch_classes_in_expr()``.
    '''
    from abjad.tools import pitchtools

    pitches = pitchtools.list_named_chromatic_pitches_in_expr(expr)
    pitch_classes = [pitchtools.NumberedChromaticPitchClass(pitch) for pitch in pitches]
    pitch_classes = tuple(pitch_classes)
    return pitch_classes



import textwrap
from abjad.tools.datastructuretools.TreeNode import TreeNode


class ReSTParagraph(TreeNode):
    '''An ReST paragraph:

    ::

        >>> paragraph = documentationtools.ReSTParagraph(text='blah blah blah')
        >>> paragraph
        ReSTParagraph(
            text='blah blah blah',
            wrap=True
            )

    ::

        >>> print _.rest_format
        blah blah blah

    Handles automatic linewrapping.

    Return `ReSTParagraph` instance.
    ''' 

    ### INITIALIZER ###

    def __init__(self, name=None, text='', wrap=True):
        TreeNode.__init__(self, name=name)
        self.text = text
        self.wrap = wrap

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _rest_format_contributions(self):
        if self.wrap:
            text = ' '.join(self.text.splitlines())
            return textwrap.wrap(text)
        return [self.text]

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def rest_format(self):
        return '\n'.join(self._rest_format_contributions)

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def text():
        def fget(self):
            return self._text
        def fset(self, arg):
            assert isinstance(arg, str)
            arg = arg.strip()
            assert len(arg)
            self._text = arg
        return property(**locals())

    @apply
    def wrap():
        def fget(self):
            return self._wrap
        def fset(self, arg):
            self._wrap = bool(arg)
        return property(**locals())




import abc
from abjad.tools.instrumenttools._Instrument import _Instrument


class _WindInstrument(_Instrument):
    '''.. versionadded:: 2.0

    Abjad model of the wind instrument.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    ### INITIALIZER ###

    def __init__(self, **kwargs):
        _Instrument.__init__(self, **kwargs)
        self._default_instrument_name = 'wind instrument'
        self._default_performer_names.append('wind player')



from abjad.tools import stringtools
from abjad.tools.lilypondproxytools.LilyPondGrobProxy import LilyPondGrobProxy


class LilyPondGrobProxyContextWrapper(object):
    '''.. versionadded:: 2.0

    Context wrapper for LilyPond grob overrides.
    '''

    ### SPECIAL METHODS ###

    def __getattr__(self, name):
        from abjad import ly
        try:
            return vars(self)[name]
        except KeyError:
            if stringtools.underscore_delimited_lowercase_to_uppercamelcase(name) in ly.grob_interfaces:
                vars(self)[name] = LilyPondGrobProxy()
                return vars(self)[name]
            else:
                raise AttributeError('object can have only LilyPond grob attributes: "%s".' %
                    self.__class__.__name__)

    def __repr__(self):
        return '%s()' % self.__class__.__name__




from abjad.tools import componenttools


def fuse_leaves_in_container_once_by_counts(container, counts, klass=None, decrease_durations_monotonically=True):
    '''Fuse leaves in `container` once by `counts` into instances of `klass`.
    '''
    from abjad.tools import containertools
    from abjad.tools import notetools
    from abjad.tools import resttools

    if klass is None:
        klass = notetools.Note

    # assert input types
    assert isinstance(container, containertools.Container)

    # assert input values
    if not sum(counts) == len(container):
        raise ValueError('sum of counts must equal length of container.')

    # find preprolated durations of glommed parts of container
    tokens = componenttools.copy_and_partition_governed_component_subtree_by_leaf_counts(
        container, counts)
    durations = [sum([x.preprolated_duration for x in part]) for part in tokens]

    # construct new notes or rests
    if klass == notetools.Note:
        new_material = notetools.make_notes(0, durations, decrease_durations_monotonically=decrease_durations_monotonically)
    elif klass == resttools.Rest:
        new_material = resttools.make_rests(durations, decrease_durations_monotonically=decrease_durations_monotonically)
    else:
        raise ValueError('unknown type of material to construct.')

    # insert new material in container
    container[:] = new_material

    # return container
    return container



from abjad.tools import componenttools
from abjad.tools import containertools


def change_tuplets_in_expr_to_fixed_duration_tuplets(expr):
    r'''.. versionadded:: 2.9

    Change tuplets in `expr` to fixed-duration tuplets::

        >>> staff = Staff(r"\times 2/3 { c'8 d'8 e'8 } \times 2/3 { c'8 d'8 e'8 }")

    ::

        >>> staff[:]
        Selection(Tuplet(2/3, [c'8, d'8, e'8]), Tuplet(2/3, [c'8, d'8, e'8]))

    ::

        >>> tuplettools.change_tuplets_in_expr_to_fixed_duration_tuplets(staff)
        [FixedDurationTuplet(1/4, [c'8, d'8, e'8]), FixedDurationTuplet(1/4, [c'8, d'8, e'8])]

    ::

        >>> for x in staff[:]:
        ...     x
        ...
        FixedDurationTuplet(1/4, [c'8, d'8, e'8])
        FixedDurationTuplet(1/4, [c'8, d'8, e'8])

    Return tuplets.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import tuplettools

    result = []
    for tuplet in iterationtools.iterate_tuplets_in_expr(expr):    
        if tuplet._class_name == 'Tuplet':
            target_duration = tuplet.preprolated_duration
            new_tuplet = tuplettools.FixedDurationTuplet(target_duration, [])
            containertools.move_parentage_children_and_spanners_from_components_to_empty_container(
                [tuplet], new_tuplet)
            result.append(new_tuplet)

    return result



import collections
from abjad.tools import contexttools
from abjad.tools import instrumenttools
from abjad.tools import scoretools
from abjad.tools import stafftools
from abjad.tools import voicetools
from abjad.tools.scoretemplatetools.ScoreTemplate import ScoreTemplate


class GroupedStavesScoreTemplate(ScoreTemplate):
    r'''.. versionadded:: 2.10

    Grouped staves score template::

        >>> template = scoretemplatetools.GroupedStavesScoreTemplate(staff_count=4)
        >>> score = template()

    ::

        >>> score
        Score-"Grouped Staves Score"<<1>>

    ::

        >>> f(score)
        \context Score = "Grouped Staves Score" <<
            \context StaffGroup = "Grouped Staves Staff Group" <<
                \context Staff = "Staff 1" {
                    \context Voice = "Voice 1" {
                    }
                }
                \context Staff = "Staff 2" {
                    \context Voice = "Voice 2" {
                    }
                }
                \context Staff = "Staff 3" {
                    \context Voice = "Voice 3" {
                    }
                }
                \context Staff = "Staff 4" {
                    \context Voice = "Voice 4" {
                    }
                }
            >>
        >>

    Return score template object.
    '''     

    ### INITIALIZER ###

    def __init__(self, staff_count=2):
        self.context_name_abbreviations = collections.OrderedDict()
        self.staff_count = staff_count

    ### SPECIAL METHODS ###

    def __call__(self):
        staves = []
        for index in range(self.staff_count):
            number = index + 1
            voice = voicetools.Voice(name='Voice {}'.format(number))
            staff = stafftools.Staff([voice], name='Staff {}'.format(number))
            staves.append(staff)
            self.context_name_abbreviations['v{}'.format(number)] = voice.name

        grouped_rhythmic_staves_staff_group = scoretools.StaffGroup(
            staves, name='Grouped Staves Staff Group')

        grouped_rhythmic_staves_score = scoretools.Score(
            [grouped_rhythmic_staves_staff_group], name='Grouped Staves Score')

        return grouped_rhythmic_staves_score



from abjad.tools import durationtools
from abjad.tools import componenttools
from abjad.tools import iterationtools
from abjad.tools import leaftools
from abjad.tools import resttools
from abjad.tools import skiptools


# TODO: make public and possibly improve function name
def _insert_measure_padding(expr, front, back, klass, splice=False):
    r'''.. versionadded:: 2.0

    Generalizes measuretools.pad_measures_in_expr_with_rests() and
    measuretools.pad_measures_in_expr_with_skips().
    '''

    if not isinstance(front, (durationtools.Duration, type(None))):
        raise ValueError

    if not isinstance(back, (durationtools.Duration, type(None))):
        raise ValueError

    if not isinstance(klass, (resttools.Rest, skiptools.Skip)):
        raise TypeError

    root = expr[0].parentage.root

    # forbid updates because
    # componenttools.extend_in_parent_of_component() calls self.stop_offset
    #root._update._forbid_component_update()
    root._update_prolated_offset_values_of_entire_score_tree_if_necessary()
    root._forbid_component_update()

    for measure in iterationtools.iterate_measures_in_expr(expr):
        if front is not None:
            start_components = componenttools.get_improper_descendents_of_component_that_start_with_component(
                measure)
            start_leaves = [x for x in start_components if isinstance(x, leaftools.Leaf)]
            for start_leaf in start_leaves:
                if splice:
                    componenttools.extend_in_parent_of_component(
                        start_leaf, [klass.__class__(front)], left=True, grow_spanners=True)
                else:
                    componenttools.extend_in_parent_of_component(
                        start_leaf, [klass.__class__(front)], left=True, grow_spanners=False)
        if back is not None:
            stop_components = componenttools.get_improper_descendents_of_component_that_stop_with_component(
                measure)
            stop_leaves = [x for x in stop_components if isinstance(x, leaftools.Leaf)]
            for stop_leaf in stop_leaves:
                if splice:
                    componenttools.extend_in_parent_of_component(
                        stop_leaf, [klass.__class__(back)], grow_spanners=True)
                else:
                    componenttools.extend_in_parent_of_component(
                        stop_leaf, [klass.__class__(back)], grow_spanners=False)

    # allow updates after all calls to spanner-growing functions are done #
    #root._update._allow_component_update()
    root._mark_entire_score_tree_for_later_update('prolated')
    root._allow_component_update()



from abjad import *


def test_TieChain_written_duration_01():

    staff = Staff("c' ~ c'16")

    assert tietools.get_tie_chain(staff[0]).written_duration == Duration(5, 16)


def test_TieChain_written_duration_02():

    staff = Staff("c'")

    assert tietools.get_tie_chain(staff[0]).written_duration == Duration(1, 4)



from abjad import *


def test_labeltools_color_chord_note_heads_in_expr_by_pitch_class_color_map_01():

    chord = Chord([12, 14, 18, 21, 23], (1, 4))

    r'''
    <c'' d'' fs'' a'' b''>4
    '''

    pitches = [[-12, -10, 4], [-2, 8, 11, 17], [19, 27, 30, 33, 37]]    
    colors = ['red', 'blue', 'green']    
    color_map = pitchtools.NumberedChromaticPitchClassColorMap(pitches, colors)
    labeltools.color_chord_note_heads_in_expr_by_pitch_class_color_map(chord, color_map)

    r'''
    <
        \tweak #'color #red
        c''
        \tweak #'color #red
        d''
        \tweak #'color #green
        fs''
        \tweak #'color #green
        a''
        \tweak #'color #blue
        b''
    >4
    '''

    assert wellformednesstools.is_well_formed_component(chord)
    assert chord.lilypond_format == "<\n\t\\tweak #'color #red\n\tc''\n\t\\tweak #'color #red\n\td''\n\t\\tweak #'color #green\n\tfs''\n\t\\tweak #'color #green\n\ta''\n\t\\tweak #'color #blue\n\tb''\n>4"



import fractions
from abjad.tools import durationtools
from abjad.tools import formattools
from abjad.tools import mathtools
from abjad.tools.containertools.Container import Container


class Tuplet(Container):
    r'''Abjad model of a tuplet:

    ::

        >>> tuplet = Tuplet(Multiplier(2, 3), "c'8 d'8 e'8")

    ::

        >>> tuplet
        Tuplet(2/3, [c'8, d'8, e'8])

    ::

        >>> f(tuplet)
        \times 2/3 {
            c'8
            d'8
            e'8
        }

    ::

        >>> show(tuplet) # doctest: +SKIP

    Tuplets, like any counttime container, may nest arbitrarily:

    ::

        >>> second_tuplet = Tuplet((4, 7), "g'4. ( a'16 )")
        >>> tuplet.insert(1, second_tuplet)

    ::

        >>> f(tuplet)
        \times 2/3 {
            c'8
            \times 4/7 {
                g'4. (
                a'16 )
            }
            d'8
            e'8
        }


    ::

        >>> show(tuplet) # doctest: +SKIP

    ::

        >>> third_tuplet = Tuplet((4, 5), "e''32 [ ef''32 d''32 cs''32 cqs''32 ]")
        >>> second_tuplet.insert(1, third_tuplet)

    ::

        >>> f(tuplet)
        \times 2/3 {
            c'8
            \times 4/7 {
                g'4. (
                \times 4/5 {
                    e''32 [
                    ef''32
                    d''32
                    cs''32
                    cqs''32 ]
                }
                a'16 )
            }
            d'8
            e'8
        }

    ::

        >>> show(tuplet) # doctest: +SKIP

    Return tuplet object.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_force_fraction', '_is_invisible', '_multiplier', '_preferred_denominator',
        '_signifier', )

    ### INITIALIZER ###

    def __init__(self, multiplier, music=None, **kwargs):
        Container.__init__(self, music)
        self.multiplier = multiplier
        self._force_fraction = None
        self._is_invisible = None
        self._preferred_denominator = None
        self._signifier = '*'
        self._initialize_keyword_values(**kwargs)

    ### SPECIAL METHODS ###

    def __add__(self, arg):
        '''Add two tuplets of same type and with same multiplier.
        '''
        from abjad.tools import tuplettools
        assert isinstance(arg, type(self))
        new = tuplettools.fuse_tuplets([self, arg])
        return new

    def __getnewargs__(self):
        return (self.multiplier, )

    def __repr__(self):
        return '%s(%s, [%s])' % (type(self).__name__, self.multiplier, self._summary)

    def __str__(self):
        if 0 < len(self):
            return '{%s %s %s %s}' % (self._signifier, self.ratio_string, self._summary, self._signifier)
        else:
            return '{%s %s %s}' % (self._signifier, self.multiplier, self._signifier)

    ### READ-ONLY PRIVATE PROPERTIES ###

    # TODO: make public
    @property
    def _is_visible(self):
        return not self.is_invisible

    @property
    def _multiplier_fraction_string(self):
        if self.preferred_denominator is not None:
            inverse_multiplier = durationtools.Multiplier(
                self.multiplier.denominator, self.multiplier.numerator)
            nonreduced_fraction = mathtools.NonreducedFraction(inverse_multiplier)
            nonreduced_fraction = nonreduced_fraction.with_denominator(self.preferred_denominator)
            d, n = nonreduced_fraction.pair
        else:
            n, d = self.multiplier.numerator, self.multiplier.denominator
        return '%s/%s' % (n, d)

    @property
    def _summary(self):
        if 0 < len(self):
            return ', '.join([str(x) for x in self._music])
        else:
            return ''

    ### PRIVATE METHODS ###

    def _format_after_slot(self, format_contributions):
        '''Read-only tuple of format contributions to appear immediately after self closing.
        '''
        result = []
        result.append(('grob reverts', format_contributions.get('grob reverts', [])))
        result.append(('lilypond command marks', 
            format_contributions.get('after', {}).get('lilypond command marks', [])))
        result.append(('comments', format_contributions.get('after', {}).get('comments', [])))
        return tuple(result)

    def _format_before_slot(self, format_contributions):
        result = []
        result.append(('comments', format_contributions.get('before', {}).get('comments', [])))
        result.append(('lilypond command marks', 
            format_contributions.get('before', {}).get('lilypond command marks', [])))
        result.append(('grob overrides', format_contributions.get('grob overrides', [])))
        return tuple(result)

    def _format_close_brackets_slot(self, format_contributions):
        '''Read-only tuple of format contributions used to generate self closing.
        '''
        result = []
        if self.multiplier:
            result.append([('self_brackets', 'close'), '}'])
        return tuple(result)

    def _format_closing_slot(self, format_contributions):
        '''Read-only tuple of format contributions to appear immediately before self closing.
        '''
        result = []
        result.append(('lilypond command marks', 
            format_contributions.get('closing', {}).get('lilypond command marks', [])))
        result.append(('comments', format_contributions.get('closing', {}).get('comments', [])))
        return self._format_slot_contributions_with_indent(result)

    def _format_lilypond_fraction_command_string(self):
        if self._is_visible:
            if self.is_augmentation or self.has_non_power_of_two_denominator or self.force_fraction:
                return r'\fraction '
        return ''

    def _format_open_brackets_slot(self, format_contributions):
        result = []
        if self.multiplier:
            if self.is_invisible:
                multiplier = self.multiplier
                n, d = multiplier.numerator, multiplier.denominator
                contributor = (self, 'is_invisible')
                contributions = [r"\scaleDurations #'(%s . %s) {" % (n, d)]
                result.append([contributor, contributions])
            else:
                contributor = ('self_brackets', 'open')
                if self.multiplier != 1:
                    contributions = [r'%s\times %s %s' % (
                        self._format_lilypond_fraction_command_string(),
                        self._multiplier_fraction_string,
                        '{'
                        )]
                else:
                    contributions = ['{']
                result.append([contributor, contributions])
        return tuple(result)

    def _format_opening_slot(self, format_contributions):
        '''Read-only tuple of format contributions to appear immediately after self opening.
        '''
        result = []
        result.append(('comments', format_contributions.get('opening', {}).get('comments', [])))
        result.append(('lilypond command marks', 
            format_contributions.get('opening', {}).get('lilypond command marks', [])))
        return self._format_slot_contributions_with_indent(result)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def has_non_power_of_two_denominator(self):
        '''Read-only boolean true when multiplier numerator is not power of two. Otherwise false::

            >>> tuplet = Tuplet((3, 5), "c'8 d'8 e'8 f'8 g'8")
            >>> tuplet.has_non_power_of_two_denominator
            True

        Return boolean.
        '''
        return not self.has_power_of_two_denominator

    @property
    def has_power_of_two_denominator(self):
        '''Read-only boolean true when multiplier numerator is power of two. Otherwise false::

            >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
            >>> tuplet.has_power_of_two_denominator
            True

        Return boolean.
        '''
        if self.multiplier:
            return mathtools.is_nonnegative_integer_power_of_two(self.multiplier.numerator)
        else:
            return True

    @property
    def implied_prolation(self):
        r'''Tuplet implied prolation.

        Defined equal to tuplet multiplier.

        Return multiplier.
        '''
        return self.multiplier

    @property
    def is_augmentation(self):
        '''True when multiplier is greater than 1.
        Otherwise false::

            >>> t = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
            >>> t.is_augmentation
            False

        Return boolean.
        '''
        if self.multiplier:
            return 1 < self.multiplier
        else:
            return False

    @property
    def is_diminution(self):
        '''True when multiplier is less than 1.  Otherwise false::

            >>> t = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
            >>> t.is_diminution
            True

        Return boolean.
        '''
        if self.multiplier:
            return self.multiplier < 1
        else:
            return False

    @property
    def is_trivial(self):
        '''True when tuplet multiplier is one. Otherwise false::

            >>> tuplet = Tuplet((1, 1), "c'8 d'8 e'8")
            >>> tuplet.is_trivial
            True
        
        Return boolean.
        '''
        return self.multiplier == 1

    @property
    def lilypond_format(self):
        self._update_marks_of_entire_score_tree_if_necessary()
        return self._format_component()

    @property
    def multiplied_duration(self):
        '''Read-only multiplied duration of tuplet::

            >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
            >>> tuplet.multiplied_duration
            Duration(1, 4)

        Return duration.
        '''
        return self.multiplier * self.contents_duration

    @property
    def preprolated_duration(self):
        '''Duration prior to prolation:

        ::

            >>> t = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
            >>> t.preprolated_duration
            Duration(1, 4)

        Return duration.
        '''
        return self.multiplied_duration

    @property
    def ratio_string(self):
        '''Read-only tuplet multiplier formatted with colon as ratio::

            >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
            >>> tuplet.ratio_string
            '3:2'

        Return string.
        '''
        multiplier = self.multiplier
        if multiplier is not None:
            return '%s:%s' % (multiplier.denominator, multiplier.numerator)
        else:
            return None

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def force_fraction():
        def fget(self):
            r'''Read / write boolean to force ``n:m`` fraction in LilyPond format::

                >>> tuplet = Tuplet(Fraction(2, 3), "c'8 d'8 e'8")

            ::

                >>> tuplet.force_fraction is None
                True

            ::

                >>> f(tuplet)
                \times 2/3 {
                    c'8
                    d'8
                    e'8
                }


            ::

                >>> tuplet.force_fraction = True

            ::

                >>> f(tuplet)
                \fraction \times 2/3 {
                    c'8
                    d'8
                    e'8
                }

            Return boolean or none.
            '''
            return self._force_fraction
        def fset(self, arg):
            if isinstance(arg, (bool, type(None))):
                self._force_fraction = arg
            else:
                raise TypeError('bad type for tuplet force fraction: "%s".' % arg)
        return property(**locals())

    @apply
    def is_invisible():
        def fget(self):
            r'''Read / write boolean to render as LilyPond ``\scaledDurations``
            construct instead of tuplet::

                >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")

            ::

                >>> f(tuplet)
                \times 2/3 {
                    c'8
                    d'8
                    e'8
                }

            ::

                >>> tuplet.is_invisible = True

            ::

            
                \scaleDurations #'(2 . 3) {
                    c'8
                    d'8
                    e'8
                }

            This has the effect of rendering no
            no tuplet bracket and no tuplet number while preserving the rhythmic
            value of the tuplet and the contents of the tuplet.

            Return boolean or none.
            '''
            return self._is_invisible
        def fset(self, arg):
            assert isinstance(arg, (bool, type(None)))
            self._is_invisible = arg
        return property(**locals())

    @apply
    def multiplier():
        def fget(self):
            r'''Read / write tuplet multiplier::

                >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
                >>> tuplet.multiplier
                Multiplier(2, 3)

            Return multiplier.
            ''' 
            return self._multiplier
        def fset(self, expr):
            if isinstance(expr, (int, long, fractions.Fraction)):
                rational = durationtools.Multiplier(expr)
            elif isinstance(expr, tuple):
                rational = durationtools.Multiplier(expr)
            else:
                raise ValueError('can not set tuplet multiplier: {!r}.'.format(expr))
            if 0 < rational:
                self._multiplier = rational
            else:
                raise ValueError('tuplet multiplier must be positive: {!r}.'.format(expr))
        return property(**locals())

    @apply
    def preferred_denominator():
        def fget(self):
            r'''.. versionadded:: 2.0

            Integer denominator in terms of which tuplet fraction should format::

                >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
                >>> tuplet.preferred_denominator = 4

            ::

                >>> f(tuplet)
                \times 4/6 {
                    c'8
                    d'8
                    e'8
                }

            Return positive integer or none.
            '''
            return self._preferred_denominator
        def fset(self, arg):
            if isinstance(arg, (int, long)):
                if not 0 < arg:
                    raise ValueError('tuplet preferred denominator must be positive: "%s".' % arg)
            elif not isinstance(arg, type(None)):
                raise TypeError('bad tuplet preferred denominator type: "%s".' % arg)
            self._preferred_denominator = arg
        return property(**locals())



# load constants into __builtins__ namespace
from abjad.tools import datastructuretools
from abjad.tools import mathtools
__builtins__['Left'] = datastructuretools.OrdinalConstant('x', -1, 'Left')
__builtins__['Right'] = datastructuretools.OrdinalConstant('x', 1, 'Right')
__builtins__['Center'] = datastructuretools.OrdinalConstant('y', 0, 'Center')
__builtins__['Up'] = datastructuretools.OrdinalConstant('y', 1, 'Up')
__builtins__['Down'] = datastructuretools.OrdinalConstant('y', -1, 'Down')
__builtins__['Infinity'] = mathtools.Infinity()
__builtins__['NegativeInfinity'] = mathtools.NegativeInfinity()

from abjad.tools import abctools
from abjad.tools import abjadbooktools
from abjad.tools import beamtools
from abjad.tools import chordtools
from abjad.tools import componenttools
from abjad.tools import configurationtools
from abjad.tools import containertools
from abjad.tools import contexttools
from abjad.tools import decoratortools
from abjad.tools import developerscripttools
from abjad.tools import documentationtools
from abjad.tools import durationtools
from abjad.tools import exceptiontools
from abjad.tools import formattools
from abjad.tools import gracetools
from abjad.tools import importtools
from abjad.tools import instrumenttools
from abjad.tools import introspectiontools
from abjad.tools import iotools
from abjad.tools import iterationtools
from abjad.tools import labeltools
from abjad.tools import layouttools
from abjad.tools import leaftools
from abjad.tools import lilypondfiletools
from abjad.tools import lilypondparsertools
from abjad.tools import lilypondproxytools
from abjad.tools import marktools
from abjad.tools import markuptools
from abjad.tools import measuretools
from abjad.tools import notetools
from abjad.tools import offsettools
from abjad.tools import pitcharraytools
from abjad.tools import pitchtools
from abjad.tools import quantizationtools
from abjad.tools import resttools
from abjad.tools import rhythmmakertools
from abjad.tools import rhythmtreetools
from abjad.tools import schemetools
from abjad.tools import scoretemplatetools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import sievetools
from abjad.tools import skiptools
from abjad.tools import spannertools
from abjad.tools import stafftools
from abjad.tools import stringtools
from abjad.tools import tempotools
from abjad.tools import testtools
from abjad.tools import tietools
from abjad.tools import timeintervaltools
from abjad.tools import timesignaturetools
from abjad.tools import timerelationtools
from abjad.tools import timespantools
from abjad.tools import tonalitytools
from abjad.tools import tuplettools
from abjad.tools import verticalitytools
from abjad.tools import voicetools
from abjad.tools import wellformednesstools



import abc
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner


class DirectedSpanner(Spanner):
    '''Abstract Spanner subclass for spanners which may take an "up" or "down"
    indication.
    '''
    
    @abc.abstractmethod
    def __init__(self, components=[], direction=None):
        Spanner.__init__(self, components)
        self.direction = direction

    ### PUBLIC PROPERTIES ###

    @apply
    def direction():
        def fget(self):
            return self._direction
        def fset(self, arg):
            self._direction = stringtools.arg_to_tridirectional_lilypond_symbol(arg)
        return property(**locals())

    ### PRIVATE METHODS ###

    @abc.abstractmethod
    def _copy_keyword_args(self, new):
        new.direction = self.direction




from abjad.tools import documentationtools
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript
import os


class MakeNewClassTemplateScript(DeveloperScript):
    '''Create class stubs, complete with test subdirectory:

    ::

        bash$ ajv new class -h
        usage: make-new-class-template [-h] [--version] (-X | -M) name

        Make a new class template file.

        positional arguments:
          name                tools package qualified class name

        optional arguments:
          -h, --help          show this help message and exit
          --version           show program's version number and exit
          -X, --experimental  use the Abjad experimental tools path
          -M, --mainline      use the Abjad mainline tools path

    Return `MakeNewClassTemplateScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'class'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return 'new'

    @property
    def short_description(self):
        return 'Make a new class template file.'

    @property
    def version(self):
        return 1.0

    ### PRIVATE METHODS ###

    def _get___init___text(self, package_root_name):
        return [
            'from abjad.tools import importtools',
            '',
            "importtools.import_structured_package(__path__[0], globals(), package_root_name={!r})".format(package_root_name),
        ]

    def _get_class_names_in_tools_package(self, root, tools_package_name):
        path = os.path.join(root, tools_package_name)
        crawler = documentationtools.ClassCrawler(path, include_private_objects=True)
        objs = crawler()
        return tuple(sorted([x.__name__ for x in objs]))

    def _get_class_text(self, class_name):
        return [
            'from abjad.tools.abctools import AbjadObject',
            '',
            '',
            'class {}(AbjadObject):'.format(class_name),
            '    pass'
        ]

    def _get_tools_package_names(self, root):
        names = []
        for x in os.listdir(root):
            if os.path.isdir(os.path.join(root, x)):
                if not x.startswith(('_', '.')):
                    names.append(x)
        return tuple(sorted(names))

    ### PUBLIC METHODS ###

    def process_args(self, args):

        from abjad import ABJCFG

        if args.name.count('.') != 1:
            print 'Error: {!r} not in tools_package.class_name format.'.format(args.name)
            return

        root = args.path
        tools_package_name, class_name = args.name.split('.')

        if tools_package_name not in self._get_tools_package_names(root):
            print 'Error: {!r} is not a valid tools package.'.format(tools_package_name)
            return

        if class_name in self._get_class_names_in_tools_package(root, tools_package_name):
            print 'Error: {!r} already exists in {!r}'.format(class_name, tools_package_name)
            return

        package_path = os.path.join(root, tools_package_name, class_name)

        os.mkdir(package_path)
        os.mkdir(os.path.join(package_path, 'test'))

        if args.path == os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'tools'):
            package_root_name = 'experimental'
        elif args.path == os.path.join(ABJCFG.ABJAD_PATH, 'tools'):
            package_root_name = 'abjad'
        else:
            raise Exception

        module_path = os.path.join(package_path, '{}.py'.format(class_name))
        module_text = '\n'.join(self._get_class_text(class_name)) + '\n'

        init_path = os.path.join(package_path, '__init__.py')
        init_text = '\n'.join(self._get___init___text(package_root_name)) + '\n'
        
        with open(module_path, 'w') as f:
            f.write(module_text)

        with open(init_path, 'w') as f:
            f.write(init_text)

    def setup_argument_parser(self, parser):

        from abjad import ABJCFG

        parser.add_argument('name',
            help='tools package qualified class name'
            )

        path_group = parser.add_mutually_exclusive_group(required=True)

        path_group.add_argument('-X', '--experimental',
            action='store_const',
            const=os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'tools'),
            dest='path',
            help='use the Abjad experimental tools path',
            )

        path_group.add_argument('-M', '--mainline',
            action='store_const',
            const=os.path.join(ABJCFG.ABJAD_PATH, 'tools'),
            dest='path',
            help='use the Abjad mainline tools path',
            )



from abjad import *
import py


def test_timesignaturetools_establish_metrical_hierarchy_01():
    source = p('abj: | 4/4 8 2. 8 |')
    target = p('abj: | 4/4 8 8 ~ 2 ~ 8 8 |')
    metrical_hierarchy = timesignaturetools.MetricalHierarchy(source)
    timesignaturetools.establish_metrical_hierarchy(source[:], metrical_hierarchy)
    assert source.lilypond_format == target.lilypond_format


def test_timesignaturetools_establish_metrical_hierarchy_02():
    '''Establishes metrical hierarchy when first component's score offset greater than zero.
    '''
    source = p('abj: | 2/4 4 4 ~ || 4/4 8 2. 8 ~ || 2/4 4 4 |')
    target = p('abj: | 2/4 4 4 ~ || 4/4 8 8 ~ 2 ~ 8 8 ~ || 2/4 4 4 |')
    metrical_hierarchy = timesignaturetools.MetricalHierarchy(source[1])
    timesignaturetools.establish_metrical_hierarchy(source[1][:], metrical_hierarchy)
    assert source.lilypond_format == target.lilypond_format


def test_timesignaturetools_establish_metrical_hierarchy_03():
    '''Descends into tuplets.'''
    source = p('abj: | 2/4 2 ~ || 5/4 8 ~ 8 ~ 2/3 { 4 ~ 4 4 ~ } 4 ~ 4 ~ || 2/4 2 |') 
    target = p('abj: | 2/4 2 ~ || 5/4 4 ~ 2/3 { 2 4 ~ } 2 ~ || 2/4 2 |')
    metrical_hierarchy = timesignaturetools.MetricalHierarchy(source[1])
    timesignaturetools.establish_metrical_hierarchy(source[1][:], metrical_hierarchy)
    assert source.lilypond_format == target.lilypond_format


def test_timesignaturetools_establish_metrical_hierarchy_04():
    source = p("abj: | 4/4 c'8. d'4.. e'4. |")
    target = p("abj: | 4/4 c'8. d'16 ~ d'4. e'4. |")
    metrical_hierarchy = timesignaturetools.MetricalHierarchy(source)
    timesignaturetools.establish_metrical_hierarchy(source[:], metrical_hierarchy)
    assert source.lilypond_format == target.lilypond_format


def test_timesignaturetools_establish_metrical_hierarchy_05():
    metrical_hierarchy = timesignaturetools.MetricalHierarchy((4, 4))
    for rhythm_number in range(8):
        # without boundary enforcement
        notes = timesignaturetools.make_gridded_test_rhythm(4, rhythm_number, denominator=4)
        measure = Measure((4, 4), notes)
        timesignaturetools.establish_metrical_hierarchy(measure[:], metrical_hierarchy)
        # with boundary enforcement
        notes = timesignaturetools.make_gridded_test_rhythm(4, rhythm_number, denominator=4)
        measure = Measure((4, 4), notes)
        timesignaturetools.establish_metrical_hierarchy(measure[:], metrical_hierarchy, boundary_depth=-1)


def test_timesignaturetools_establish_metrical_hierarchy_06():
    source = p('abj: | 4/4 8 4. 2 |')
    target = p('abj: | 4/4 8 4. 2 |')
    metrical_hierarchy = timesignaturetools.MetricalHierarchy((4, 4))
    timesignaturetools.establish_metrical_hierarchy(source[:], metrical_hierarchy)
    assert source.lilypond_format == target.lilypond_format


def test_timesignaturetools_establish_metrical_hierarchy_07():
    '''Can limit dot count.'''

    metrical_hierarchy = '(4/4 (1/4 1/4 1/4 1/4))'

    maximum_dot_count = None
    source = p('abj: | 4/4 2... 16 |')
    target = p('abj: | 4/4 2... 16 |')
    timesignaturetools.establish_metrical_hierarchy(source[:], metrical_hierarchy,
        maximum_dot_count=maximum_dot_count)
    assert source.lilypond_format == target.lilypond_format

    maximum_dot_count = 3
    source = p('abj: | 4/4 2... 16 |')
    target = p('abj: | 4/4 2... 16 |')
    timesignaturetools.establish_metrical_hierarchy(source[:], metrical_hierarchy,
        maximum_dot_count=maximum_dot_count)
    assert source.lilypond_format == target.lilypond_format

    maximum_dot_count = 2
    source = p('abj: | 4/4 2... 16 |')
    target = p('abj: | 4/4 2. ~ 8. 16 |')
    timesignaturetools.establish_metrical_hierarchy(source[:], metrical_hierarchy,
        maximum_dot_count=maximum_dot_count)
    assert source.lilypond_format == target.lilypond_format

    maximum_dot_count = 1
    source = p('abj: | 4/4 2... 16 |')
    target = p('abj: | 4/4 2. ~ 8. 16 |')
    timesignaturetools.establish_metrical_hierarchy(source[:], metrical_hierarchy,
        maximum_dot_count=maximum_dot_count)
    assert source.lilypond_format == target.lilypond_format

    maximum_dot_count = 0
    source = p('abj: | 4/4 2... 16 |')
    target = p('abj: | 4/4 2 ~ 4 ~ 8 ~ 16 16 |')
    timesignaturetools.establish_metrical_hierarchy(source[:], metrical_hierarchy,
        maximum_dot_count=maximum_dot_count)
    assert source.lilypond_format == target.lilypond_format



from abjad.tools import schemetools


def make_floating_time_signature_lilypond_file(music=None):
    r'''.. versionadded:: 2.10

    Make floating time signature LilyPond file from `music`.

    Function creates a basic LilyPond file.

    Function then applies many layout settings.

    View source here for the complete inventory of settings applied.

    Returns LilyPond file object.
    '''
    # TODO: lilypondfiletools should depend on layouttools and NOT the other way around
    from abjad.tools import layouttools
    from abjad.tools import lilypondfiletools

    lilypond_file = lilypondfiletools.make_basic_lilypond_file(music=music)

    lilypond_file.default_paper_size = 'letter', 'portrait'
    lilypond_file.global_staff_size = 12

    lilypond_file.paper_block.left_margin = 20
    lilypond_file.paper_block.system_system_spacing = layouttools.make_spacing_vector(0, 0, 12, 0)

    lilypond_file.layout_block.indent = 0
    lilypond_file.layout_block.ragged_right = True

    time_signature_context_block = lilypondfiletools.make_time_signature_context_block(font_size=1, padding=6)
    lilypond_file.layout_block.context_blocks.append(time_signature_context_block)

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    lilypond_file.score = context_block
    context_block.context_name = 'Score'
    context_block.accepts.append('TimeSignatureContext')
    context_block.engraver_removals.append('Bar_number_engraver')
    context_block.override.beam.breakable = True
    context_block.override.spacing_spanner.strict_grace_spacing = True
    context_block.override.spacing_spanner.strict_note_spacing = True
    context_block.override.spacing_spanner.uniform_stretching = True
    context_block.override.tuplet_bracket.bracket_visibility = True
    context_block.override.tuplet_bracket.padding = 2
    context_block.override.tuplet_bracket.springs_and_rods = schemetools.Scheme(
        'ly:spanner::set-spacing-rods')
    context_block.override.tuplet_bracket.minimum_length = 3
    context_block.override.tuplet_number.text = schemetools.Scheme('tuplet-number::calc-fraction-text')
    context_block.set.autoBeaming = False
    context_block.set.proportionalNotationDuration = schemetools.SchemeMoment((1, 32))
    context_block.set.tupletFullLength = True

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    context_block.context_name = 'Staff'
    context_block.engraver_removals.append('Time_signature_engraver')

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    context_block.context_name = 'RhythmicStaff'
    context_block.engraver_removals.append('Time_signature_engraver')

    return lilypond_file



import copy
from abjad.tools import notetools
from abjad.tools import tietools


def edit_first_violin_voice(score, durated_reservoir):

    voice = score['First Violin Voice']
    descents = durated_reservoir['First Violin']

    copied_descent = copy.deepcopy(descents[-1])
    voice.extend(copied_descent)

    final_sustain_rhythm = [(6, 4)] * 43 + [(1, 2)]
    final_sustain_notes = notetools.make_notes(["c'"], final_sustain_rhythm)
    voice.extend(final_sustain_notes)
    tietools.TieSpanner(final_sustain_notes)
    voice.extend('r4 r2.')
